@@ -2,7 +2,7 @@
                           cconfig.cpp  -  description
                              -------------------
     begin                : Tue May 14 2002
-    copyright            : (C) 2002-2003 by Mathias Kster
+    copyright            : (C) 2002-2005 by Mathias Kster
     email                : mathen@users.berlios.de
  ***************************************************************************/
 
@@ -98,6 +98,7 @@
 #define XML_MEDIUM		"medium"
 #define XML_MEDIUM_FILE		"file"
 #define XML_MEDIUM_BUFFER	"buffer"
+#define XML_HASH		"hash"
 
 #define XML_FILECHUNK		"filechunk"
 #define XML_REFCOUNT		"refcount"
@@ -132,7 +133,7 @@
 #define XML_CHECKPRIVATEADDRESSSPACE	"checkprivateaddressspace"
 
 #define XML_SEARCH_HISTORY		"searchhistory"
-#define XML_DATADIR			"datadir"
+#define XML_DCLIB_DATAPATH		"dclibdatapath"
 #define XML_DYNAMICUPLOADRATE		"dynamicuploadrate"
 
 #define XML_CHATSENDOFFLINEMESSAGES	"chatsendofflinemessages"
@@ -154,6 +155,8 @@
 #define XML_RELOAD_HUBLIST_TIME		"reloadhublisttime"
 #define XML_RECREATE_SHARELIST_TIME	"recreatesharelisttime"
 
+#define XML_DISABLEHASHLIST		"disablehashlist"
+
 #define XML_TRAFFIC_RX			"trafficrx"
 #define XML_TRAFFIC_TX			"traffictx"
 #define XML_TRAFFIC_DATA_RX		"trafficdatarx"
@@ -192,11 +195,7 @@
 
 /** default hub url list */
 const char * huburl_list[] = {
-#ifdef HAVE_LIBBZ2
-	"http://dcgui.berlios.de/PublicHubList.config.bz2",
-#else
-	"http://dcgui.berlios.de/PublicHubList.config",
-#endif
+	"http://www.hublist.org/PublicHubList.config.bz2",
 //	"http://www.neo-modus.com/PublicHubList.config",
 //	"http://dcplusplus.sourceforge.net/PublicHubList.config.bz2",
 //	"http://axljab.homelinux.org:8080/PublicHubList.config.bz2",
@@ -250,7 +249,7 @@ CConfig::CConfig( CString configpath )
 	sSearchNick    = "nick_search";
 	sAwayMessage   = "";
 	sDescription   = "http://dcgui.berlios.de";
-	sSpeed         = "";
+	sSpeed         = "28.8Kbps";
 	m_sHost        = "ppp0";
 	m_bExternalIP  = FALSE;	
 	m_sHostCache   = "";
@@ -307,8 +306,8 @@ CConfig::CConfig( CString configpath )
 	m_nBookHubID   = 0;
 
 #ifndef WIN32
-	sDatadir  = DATADIR;
-	sDatadir += "/dcgui";
+	m_sDCLibDataPath  = DCLIB_DATAPATH;
+	m_sDCLibDataPath += "/dclib";
 #else
 	// read datapath from registrie
 	LONG Status;
@@ -316,11 +315,11 @@ CConfig::CConfig( CString configpath )
 	UCHAR buffer[1024];
 	DWORD dim = 1024;
 
-	sDatadir = "";
+	m_sDCLibDataPath = "";
 
 	// load from HKLM
 	Status = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
-		TEXT("Software\\DCGUI"),
+		TEXT("Software\\VALKNUT"),
 		0,
 		KEY_READ,
 		&regdcgui);
@@ -329,7 +328,7 @@ CConfig::CConfig( CString configpath )
 	{
 		// load from HKCU
 		Status = RegOpenKeyEx(HKEY_CURRENT_USER,
-			TEXT("Software\\DCGUI"),
+			TEXT("Software\\VALKNUT"),
 			0,
 			KEY_READ,
 			&regdcgui);
@@ -341,7 +340,7 @@ CConfig::CConfig( CString configpath )
 
 		if ( Status == ERROR_SUCCESS )
 		{
-			sDatadir = (char*)buffer;
+			m_sDCLibDataPath = (char*)buffer;
 		}
 
 		RegCloseKey(regdcgui);
@@ -352,6 +351,7 @@ CConfig::CConfig( CString configpath )
 	m_nReloadHubListTime = 0;
 	m_nRecreateShareListTime = 0;
 	m_bCreateFile = FALSE;
+	m_bDisableHashList = FALSE;
 
 	SetInstance(this);
 }
@@ -546,13 +546,9 @@ int CConfig::SaveDCLib()
 
 	while( (hublisturl=pHubListUrlList.Next(hublisturl)) != 0 )
 	{
-		// filter old urls
-		if ( hublisturl->sUrl.ToUpper() != CString("http://axljab.homelinux.org:8080/PublicHubList.config.bz2").ToUpper() )
-		{
-			node1 = xmlNewChild( node, 0, (const xmlChar*)XML_URL, 0 );
-			xml->xmlNewStringChild( node1, 0, (const xmlChar*)XML_NAME, hublisturl->sUrl );
-			xml->xmlNewBoolChild( node1, 0, (const xmlChar*)XML_ENABLED, hublisturl->bEnabled );
-		}
+		node1 = xmlNewChild( node, 0, (const xmlChar*)XML_URL, 0 );
+		xml->xmlNewStringChild( node1, 0, (const xmlChar*)XML_NAME, hublisturl->sUrl );
+		xml->xmlNewBoolChild( node1, 0, (const xmlChar*)XML_ENABLED, hublisturl->bEnabled );
 	}
 
 	// other
@@ -562,12 +558,13 @@ int CConfig::SaveDCLib()
 	xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_RECONNECTTIMEOUT, CString().setNum(iReconnectTimeout) );
 	xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_TRANSFERRESPONSETIMEOUT, CString().setNum(iTransferResponseTimeout) );
 	xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_TRANSFERRESENDTIMEOUT, CString().setNum(iTransferResendTimeout) );
-	xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_DATADIR, sDatadir );
+	xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_DCLIB_DATAPATH, m_sDCLibDataPath );
 	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_CHATSENDOFFLINEMESSAGES, bChatSendOfflineMessages );
 	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_CHATRECVOFFLINEMESSAGES, bChatRecvOfflineMessages );
 	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_HUBLISTSTORELOCAL, bHubListStoreLocal );
 	xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_RELOAD_HUBLIST_TIME, CString().setNum(m_nReloadHubListTime) );
 	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_FORCEMOVEENABLED, bForceMoveEnabled );
+	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_DISABLEHASHLIST, m_bDisableHashList );
 
 	// search history
 	node = xmlNewChild( doc->children, 0, (const xmlChar*)XML_SEARCH_HISTORY, 0 );
@@ -582,7 +579,7 @@ int CConfig::SaveDCLib()
 	// save file
 	s = sConfigPath + DCLIB_CONFIG;
 
-	if ( xmlSaveFormatFile(s.Data(),doc,1) == -1 )
+	if ( xmlSaveFormatFileEnc(s.Data(),doc,"utf-8",1) == -1 )
 	{
 		err = -1;
 	}
@@ -643,9 +640,9 @@ int CConfig::LoadDCHub()
 		DCConfigHubItem *hubitem = new DCConfigHubItem();
 
 		hubitem->m_nID          = 1;
-		hubitem->m_sName        = "DC-GUI Chat";
-		hubitem->m_sHost        = "dcgui.homelinux.org:2000";
-		hubitem->m_sDescription = "DC-GUI Chat";
+		hubitem->m_sName        = "Valknut/DCLib Chat";
+		hubitem->m_sHost        = "valknut.dyndns.info:59176";
+		hubitem->m_sDescription = "Valknut/DCLib Chat";
 
 		pBookmarkHubList->Add(hubitem->m_sName,hubitem);
 		pBookmarkHubList_HostIndex->Add(hubitem->m_sHost.ToUpper(),new CString(hubitem->m_sName));
@@ -707,7 +704,7 @@ int CConfig::SaveDCPublicHub()
 	// save file
 	s = sConfigPath + DCHUB_CONFIG;
 
-	if ( xmlSaveFormatFile(s.Data(),doc,1) == -1 )
+	if ( xmlSaveFormatFileEnc(s.Data(),doc,"utf-8",1) == -1 )
 	{
 		err = -1;
 	}
@@ -753,7 +750,7 @@ int CConfig::SaveDCBookHub()
 	// save file
 	s = sConfigPath + DCBOOKHUB_CONFIG;
 
-	if ( xmlSaveFormatFile(s.Data(),doc,1) == -1 )
+	if ( xmlSaveFormatFileEnc(s.Data(),doc,"utf-8",1) == -1 )
 	{
 		err = -1;
 	}
@@ -769,7 +766,7 @@ int CConfig::LoadDCTra( CStringList * queue, CStringList * chunks )
 {
 	CXml * xml;
 	xmlNodePtr node,c1,c2,c3;
-	CString s;
+	CString s, xml_name1, xml_name2;
 	CStringList * sl1,*sl2;
 	DCHubObject * HubObject;
 
@@ -797,62 +794,67 @@ int CConfig::LoadDCTra( CStringList * queue, CStringList * chunks )
 
 					for(c2=c1->xmlChildrenNode;c2!=0;c2=c2->next)
 					{
-						if ( xml->name(c2) == XML_NICK )
+						xml_name1 = xml->name(c2);
+						if ( xml_name1 == XML_NICK )
 							TransferObject->sNick = xml->content(c2);
-						else if ( xml->name(c2) == XML_HUBNAME )
+						else if ( xml_name1 == XML_HUBNAME )
 							TransferObject->sHubName = xml->content(c2);
-						else if ( xml->name(c2) == XML_HUBHOST )
+						else if ( xml_name1 == XML_HUBHOST )
 							TransferObject->sHubHost = xml->content(c2);
-						else if ( xml->name(c2) == XML_HUB )
+						else if ( xml_name1 == XML_HUB )
 						{
 							HubObject = new DCHubObject();
 
 							for(c3=c2->xmlChildrenNode;c3!=0;c3=c3->next)
 							{
-								if ( xml->name(c3) == XML_HUBNAME )
+								xml_name2 = xml->name(c3);
+								if ( xml_name2 == XML_HUBNAME )
 									HubObject->m_sHubName = xml->content(c3);
-								else if ( xml->name(c3) == XML_HUBHOST )
+								else if ( xml_name2 == XML_HUBHOST )
 									HubObject->m_sHubHost = xml->content(c3);
-								if ( xml->name(c3) == XML_ACTIVE )
+								else if ( xml_name2 == XML_ACTIVE )
 									HubObject->m_bActive = xml->getBoolChild(c3);
 							}
 
 							TransferObject->pHubList.Add(HubObject);
 						}
-						else if ( xml->name(c2) == XML_FILE )
+						else if ( xml_name1 == XML_FILE )
 						{
 							DCTransferFileObject * TransferFileObject = new DCTransferFileObject();
 
 							for(c3=c2->xmlChildrenNode;c3!=0;c3=c3->next)
 							{
-								if ( xml->name(c3) == XML_ID )
+								xml_name2 = xml->name(c3);
+								if ( xml_name2 == XML_ID )
 									TransferFileObject->m_nID = xml->content(c3).asULL();
-								if ( xml->name(c3) == XML_TIME )
+								else if ( xml_name2 == XML_TIME )
 									TransferFileObject->m_nTime = xml->content(c3).asULL();
-								else if ( xml->name(c3) == XML_REMOTEFILE )
+								else if ( xml_name2 == XML_REMOTEFILE )
 									TransferFileObject->m_sRemoteFile = xml->content(c3);
-								else if ( xml->name(c3) == XML_LOCALFILENAME )
+								else if ( xml_name2 == XML_LOCALFILENAME )
 									TransferFileObject->m_sLocalFileName = xml->content(c3);
-								else if ( xml->name(c3) == XML_SIZE )
+								else if ( xml_name2 == XML_SIZE )
 									TransferFileObject->m_nSize = xml->content(c3).asULL();
-								else if ( xml->name(c3) == XML_PRIORITY )
+								else if ( xml_name2 == XML_PRIORITY )
 									TransferFileObject->m_nPriority = xml->content(c3).asINT();
-								else if ( xml->name(c3) == XML_STATE )
+								else if ( xml_name2 == XML_STATE )
 									TransferFileObject->m_eState = (eTransferFileState)xml->content(c3).asINT();
-								else if ( xml->name(c3) == XML_LOCALPATH )
+								else if ( xml_name2 == XML_LOCALPATH )
 									TransferFileObject->m_sLocalPath = xml->content(c3);
-								else if ( xml->name(c3) == XML_LOCALFILE )
+								else if ( xml_name2 == XML_LOCALFILE )
 									TransferFileObject->m_sLocalFile = xml->content(c3);
-								else if ( xml->name(c3) == XML_TEMPHASH )
+								else if ( xml_name2 == XML_TEMPHASH )
 									TransferFileObject->m_stHash = xml->content(c3);
-								else if ( xml->name(c3) == XML_MEDIUM )
+								else if ( xml_name2 == XML_HASH )
+									TransferFileObject->m_sHash = xml->content(c3);
+								else if ( xml_name2 == XML_MEDIUM )
 								{
 									if ( xml->content(c3) == XML_MEDIUM_BUFFER )
 										TransferFileObject->m_eMedium = eltBUFFER;
 									else
 										TransferFileObject->m_eMedium = eltFILE;
 								}
-								else if ( xml->name(c3) == XML_MULTI )
+								else if ( xml_name2 == XML_MULTI )
 									TransferFileObject->m_bMulti = xml->getBoolChild(c3);
 							}
 
@@ -894,19 +896,22 @@ int CConfig::LoadDCTra( CStringList * queue, CStringList * chunks )
 
 					for(c2=c1->xmlChildrenNode;c2!=0;c2=c2->next)
 					{
-						if ( xml->name(c2) == XML_LOCALFILE )
+						xml_name2 = xml->name(c2);
+						if ( xml_name2 == XML_LOCALFILE )
 							FileChunkObject->m_sLocalFile = xml->content(c2);
-						else if ( xml->name(c2) == XML_TEMPHASH )
+						else if ( xml_name2 == XML_TEMPHASH )
 							FileChunkObject->m_stHash = xml->content(c2);
-						else if ( xml->name(c2) == XML_SIZE )
+						else if ( xml_name2 == XML_HASH )
+							FileChunkObject->m_sHash = xml->content(c2);
+						else if ( xml_name2 == XML_SIZE )
 							FileChunkObject->m_nSize = xml->content(c2).asULL();
-						else if ( xml->name(c2) == XML_SIZEDONE )
+						else if ( xml_name2 == XML_SIZEDONE )
 							FileChunkObject->m_nSizeDone = xml->content(c2).asULL();
-						else if ( xml->name(c2) == XML_REFCOUNT )
+						else if ( xml_name2 == XML_REFCOUNT )
 							FileChunkObject->m_nReferenceCount = xml->content(c2).asINT();
-						else if ( xml->name(c2) == XML_MULTI )
+						else if ( xml_name2 == XML_MULTI )
 							FileChunkObject->m_bMulti = xml->getBoolChild(c2);
-						else if ( xml->name(c2) == XML_CHUNK )
+						else if ( xml_name2 == XML_CHUNK )
 						{
 							DCChunkObject * ChunkObject = new DCChunkObject();
 
@@ -972,9 +977,9 @@ int CConfig::SaveDCTra( CStringList * queue, CStringList * chunks )
 			node = xmlNewChild( doc->children, 0, (const xmlChar*)XML_TRANSFER, 0 );
 
 			// store nick,hubhost and hubname
-			xmlNewChild( node, 0, (const xmlChar*)XML_NICK,      (const xmlChar*) xml->ToUTF8(TransferObject->sNick).Data());
-			xmlNewChild( node, 0, (const xmlChar*)XML_HUBNAME,   (const xmlChar*) xml->ToUTF8(TransferObject->sHubName).Data());
-			xmlNewChild( node, 0, (const xmlChar*)XML_HUBHOST,   (const xmlChar*) xml->ToUTF8(TransferObject->sHubHost).Data());
+			xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_NICK,      TransferObject->sNick);
+			xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_HUBNAME,   TransferObject->sHubName);
+			xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_HUBHOST,   TransferObject->sHubHost);
 
 			HubObject = 0;
 
@@ -982,8 +987,8 @@ int CConfig::SaveDCTra( CStringList * queue, CStringList * chunks )
 			{
 				node1 = xmlNewChild( node, 0, (const xmlChar*)XML_HUB, 0 );
 
-				xmlNewChild( node1, 0, (const xmlChar*)XML_HUBNAME, (const xmlChar*) xml->ToUTF8(HubObject->m_sHubName).Data());
-				xmlNewChild( node1, 0, (const xmlChar*)XML_HUBHOST, (const xmlChar*) xml->ToUTF8(HubObject->m_sHubHost).Data());
+				xml->xmlNewStringChild( node1, 0, (const xmlChar*)XML_HUBNAME, HubObject->m_sHubName);
+				xml->xmlNewStringChild( node1, 0, (const xmlChar*)XML_HUBHOST, HubObject->m_sHubHost);
 				xml->xmlNewBoolChild( node1, 0, (const xmlChar*)XML_ACTIVE, HubObject->m_bActive );
 			}
 
@@ -995,21 +1000,22 @@ int CConfig::SaveDCTra( CStringList * queue, CStringList * chunks )
 
 				xmlNewChild( node1, 0, (const xmlChar*)XML_ID, (const xmlChar*) CString().setNum(TransferFileObject->m_nID).Data());
 				xmlNewChild( node1, 0, (const xmlChar*)XML_TIME, (const xmlChar*) CString().setNum(TransferFileObject->m_nTime).Data());				
-				xmlNewChild( node1, 0, (const xmlChar*)XML_REMOTEFILE, (const xmlChar*) xml->ToUTF8(TransferFileObject->m_sRemoteFile).Data());
-				xmlNewChild( node1, 0, (const xmlChar*)XML_LOCALFILENAME, (const xmlChar*) xml->ToUTF8(TransferFileObject->m_sLocalFileName).Data());
+				xml->xmlNewStringChild( node1, 0, (const xmlChar*)XML_REMOTEFILE, TransferFileObject->m_sRemoteFile);
+				xml->xmlNewStringChild( node1, 0, (const xmlChar*)XML_LOCALFILENAME, TransferFileObject->m_sLocalFileName);
 				xmlNewChild( node1, 0, (const xmlChar*)XML_SIZE,(const xmlChar*) CString().setNum(TransferFileObject->m_nSize).Data());
 				xmlNewChild( node1, 0, (const xmlChar*)XML_PRIORITY,(const xmlChar*) CString().setNum(TransferFileObject->m_nPriority).Data());
 				xmlNewChild( node1, 0, (const xmlChar*)XML_STATE,(const xmlChar*) CString().setNum(TransferFileObject->m_eState).Data());
-				xmlNewChild( node1, 0, (const xmlChar*)XML_LOCALFILE, (const xmlChar*) xml->ToUTF8(TransferFileObject->m_sLocalFile).Data());
-				xmlNewChild( node1, 0, (const xmlChar*)XML_LOCALPATH, (const xmlChar*) xml->ToUTF8(TransferFileObject->m_sLocalPath).Data());
+				xml->xmlNewStringChild( node1, 0, (const xmlChar*)XML_LOCALFILE, TransferFileObject->m_sLocalFile);
+				xml->xmlNewStringChild( node1, 0, (const xmlChar*)XML_LOCALPATH, TransferFileObject->m_sLocalPath);
 
 				xml->xmlNewStringChild( node1, 0, (const xmlChar*)XML_TEMPHASH, TransferFileObject->m_stHash, FALSE );
-
+				xml->xmlNewStringChild( node1, 0, (const xmlChar*)XML_HASH, TransferFileObject->m_sHash, FALSE );
+				
 				if(TransferFileObject->m_eMedium==eltBUFFER)
 					s = XML_MEDIUM_BUFFER;
 				else
 					s = XML_MEDIUM_FILE;			
-				xmlNewChild( node1, 0, (const xmlChar*)XML_MEDIUM, (const xmlChar*) xml->ToUTF8(s).Data());
+				xml->xmlNewStringChild( node1, 0, (const xmlChar*)XML_MEDIUM, s );
 
 				xml->xmlNewBoolChild( node1, 0, (const xmlChar*)XML_MULTI, TransferFileObject->m_bMulti );
 			}
@@ -1021,8 +1027,9 @@ int CConfig::SaveDCTra( CStringList * queue, CStringList * chunks )
 	{
 		node = xmlNewChild( doc->children, 0, (const xmlChar*)XML_FILECHUNK, 0 );
 
-		xmlNewChild( node, 0, (const xmlChar*)XML_LOCALFILE, (const xmlChar*) xml->ToUTF8(FileChunkObject->m_sLocalFile).Data());
-		xmlNewChild( node, 0, (const xmlChar*)XML_TEMPHASH,(const xmlChar*) xml->ToUTF8(FileChunkObject->m_stHash).Data());
+		xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_LOCALFILE, FileChunkObject->m_sLocalFile );
+		xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_TEMPHASH, FileChunkObject->m_stHash );
+		xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_HASH, FileChunkObject->m_sHash );
 		xmlNewChild( node, 0, (const xmlChar*)XML_SIZE,(const xmlChar*) CString().setNum(FileChunkObject->m_nSize).Data());
 		xmlNewChild( node, 0, (const xmlChar*)XML_SIZEDONE,(const xmlChar*) CString().setNum(FileChunkObject->m_nSizeDone).Data());
 		xmlNewChild( node, 0, (const xmlChar*)XML_REFCOUNT,(const xmlChar*) CString().setNum(FileChunkObject->m_nReferenceCount).Data());
@@ -1040,7 +1047,7 @@ int CConfig::SaveDCTra( CStringList * queue, CStringList * chunks )
 
 	s = sConfigPath + DCTRA_CONFIG;
 
-	if ( xmlSaveFormatFile(s.Data(),doc,1) == -1 )
+	if ( xmlSaveFormatFileEnc(s.Data(),doc,"utf-8",1) == -1 )
 	{
 		err = -1;
 	}
@@ -1145,7 +1152,7 @@ void CConfig::ParseDCHubConfig( xmlNodePtr node )
 						reorder = TRUE;
 					}
 
-					if ( hubitem->m_nID > m_nPublicHubID )
+					else if ( hubitem->m_nID > m_nPublicHubID )
 					{
 						m_nPublicHubID = hubitem->m_nID;
 					}
@@ -1232,7 +1239,7 @@ void CConfig::ParseDCBookHubConfig( xmlNodePtr node )
 						reorder = TRUE;
 					}
 
-					if ( hubitem->m_nID > m_nBookHubID )
+					else if ( hubitem->m_nID > m_nBookHubID )
 					{
 						m_nBookHubID = hubitem->m_nID;
 					}
@@ -1278,77 +1285,76 @@ void CConfig::ParseDCLibConfig( xmlNodePtr node )
 {
 	CXml * xml;
 	xmlNodePtr n1,n2,n3;
-	CString xml_name;
+	CString xml_name1, xml_name2;
 
 	xml = new CXml();
 
 	for(n1=node;n1!=0;n1=n1->next)
 	{
+		xml_name1 = xml->name(n1);
 		/* parse identify entrys */
-		if ( xml->name(n1) == XML_IDENTIFY )
+		if ( xml_name1 == XML_IDENTIFY )
 		{
 			for(n2=n1->xmlChildrenNode;n2!=0;n2=n2->next)
 			{
-				xml_name = xml->name(n2);
+				xml_name2 = xml->name(n2);
 
-				if ( xml_name == XML_NICK )
+				if ( xml_name2 == XML_NICK )
 					sNick = xml->content(n2);
-				else if ( xml_name == XML_SEARCHNICK )
+				else if ( xml_name2 == XML_SEARCHNICK )
 					sSearchNick = xml->content(n2);
-				else if ( xml_name == XML_AWAYMESSAGE )
+				else if ( xml_name2 == XML_AWAYMESSAGE )
 					sAwayMessage = xml->content(n2);
-				else if ( xml_name == XML_EMAIL )
+				else if ( xml_name2 == XML_EMAIL )
 					sEMail = xml->content(n2);
-				else if ( xml_name == XML_ANTISPAM )
+				else if ( xml_name2 == XML_ANTISPAM )
 					bAntiSpam = xml->getBoolChild(n2);
-				else if ( xml_name == XML_DESCRIPTION )
+				else if ( xml_name2 == XML_DESCRIPTION )
 					sDescription = xml->content(n2);
-				else if ( xml_name == XML_AWAY_PREFIX )
+				else if ( xml_name2 == XML_AWAY_PREFIX )
 					sAwayPrefix = xml->content(n2);
-				else if ( xml_name == XML_DESCRIPTION_TAG )
+				else if ( xml_name2 == XML_DESCRIPTION_TAG )
 					bDescriptionTag = xml->getBoolChild(n2);
-				else if ( xml_name == XML_SPEED )
+				else if ( xml_name2 == XML_SPEED )
 					sSpeed = xml->content(n2);
 			}
 		}
 		/* parse logfile entry */
-		else if ( xml->name(n1) == XML_LOGFILE )
+		else if ( xml_name1 == XML_LOGFILE )
 		{
 			for(n2=n1->xmlChildrenNode;n2!=0;n2=n2->next)
 			{
-				xml_name = xml->name(n2);
+				xml_name2 = xml->name(n2);
 
-				if ( xml_name == XML_LOGFILENAME )
+				if ( xml_name2 == XML_LOGFILENAME )
 					sLogFile = xml->content(n2);
-				else if ( xml_name == XML_LOGFILEON )
+				else if ( xml_name2 == XML_LOGFILEON )
 					bLogFile = xml->getBoolChild(n2);
 			}
 		}
-		else if ( xml->name(n1) == XML_SECURITY )
+		else if ( xml_name1 == XML_SECURITY )
 		{
 			for(n2=n1->xmlChildrenNode;n2!=0;n2=n2->next)
 			{
-				xml_name = xml->name(n2);
-
-				if ( xml_name == XML_FLOODOPKICKMESSAGE )
+				if ( xml->name(n2) == XML_FLOODOPKICKMESSAGE )
 					sFloodOpKickMessage = xml->content(n2);
 			}
 		
 		}
 		/* parse transfer entrys */
-		else if ( xml->name(n1) == XML_TRANSFER )
+		else if ( xml_name1 == XML_TRANSFER )
 		{
 			for(n2=n1->xmlChildrenNode;n2!=0;n2=n2->next)
 			{
-				xml_name = xml->name(n2);
+				xml_name2 = xml->name(n2);
 
-				if ( xml_name == XML_DOWNLOADFOLDER )
+				if ( xml_name2 == XML_DOWNLOADFOLDER )
 					sDownloadFolder = xml->content(n2);
-				else if ( xml_name == XML_DOWNLOADFINISHEDFOLDER )
+				else if ( xml_name2 == XML_DOWNLOADFINISHEDFOLDER )
 					sDownloadFinishedFolder = xml->content(n2);
-				else if ( xml_name == XML_TRANSFER_AUTOSEARCH )
+				else if ( xml_name2 == XML_TRANSFER_AUTOSEARCH )
 					bTransferAutoSearch = xml->getBoolChild(n2);
-				else if ( xml_name == XML_SHAREDFOLDER )
+				else if ( xml_name2 == XML_SHAREDFOLDER )
 				{
 					DCConfigShareFolder * csf = new DCConfigShareFolder();
 
@@ -1365,121 +1371,122 @@ void CConfig::ParseDCLibConfig( xmlNodePtr node )
 					else
 						SharedFolders.Add( csf );
 				}
-				else if ( xml_name == XML_AUTORECREATESHARELIST )
+				else if ( xml_name2 == XML_AUTORECREATESHARELIST )
 					bAutoRecreateShareList = xml->getBoolChild(n2);
-				else if ( xml_name == XML_RECREATE_SHARELIST_TIME )
+				else if ( xml_name2 == XML_RECREATE_SHARELIST_TIME )
 					m_nRecreateShareListTime = xml->content(n2).asINT();
-				else if ( xml_name == XML_MAXUPLOAD )
+				else if ( xml_name2 == XML_MAXUPLOAD )
 					iMaxUpload = xml->content(n2).asINT();
-				else if ( xml_name == XML_USER_UPLOAD_SLOTS )
+				else if ( xml_name2 == XML_USER_UPLOAD_SLOTS )
 					iUserUploadSlots = xml->content(n2).asINT();
-				else if ( xml_name == XML_MAXUPLOADRATE )
+				else if ( xml_name2 == XML_MAXUPLOADRATE )
 					lMaxUploadRate = xml->content(n2).asULL();
-				else if ( xml_name == XML_MAXDOWNLOADRATE )
+				else if ( xml_name2 == XML_MAXDOWNLOADRATE )
 					lMaxDownloadRate = xml->content(n2).asULL();
-				else if ( xml_name == XML_DOWNLOADQUEUETIME )
+				else if ( xml_name2 == XML_DOWNLOADQUEUETIME )
 					iDownloadQueueTime = xml->content(n2).asINT();
-				else if ( xml_name == XML_DYNAMICUPLOADRATE )
+				else if ( xml_name2 == XML_DYNAMICUPLOADRATE )
 					bDynamicUploadRate = xml->getBoolChild(n2);
-				else if ( xml_name == XML_TRANSFER_CERT )
+				else if ( xml_name2 == XML_TRANSFER_CERT )
 					m_sTransferCert = xml->content(n2);
-				else if ( xml_name == XML_TRANSFER_KEY )
+				else if ( xml_name2 == XML_TRANSFER_KEY )
 					m_sTransferKey = xml->content(n2);
-				else if ( xml_name == XML_HUBOFFLINE_TRANSFERCLOSE )
+				else if ( xml_name2 == XML_HUBOFFLINE_TRANSFERCLOSE )
 					eHubOfflineTransferClose = (eCloseType)xml->content(n2).asINT();
-				else if ( xml_name == XML_TRAFFIC_RX )
+				else if ( xml_name2 == XML_TRAFFIC_RX )
 					CSocket::m_Traffic.AddTraffic( ettRX, xml->content(n2).asULL() );
-				else if ( xml_name == XML_TRAFFIC_TX )
+				else if ( xml_name2 == XML_TRAFFIC_TX )
 					CSocket::m_Traffic.AddTraffic( ettTX, xml->content(n2).asULL() );
-				else if ( xml_name == XML_TRAFFIC_DATA_RX )
+				else if ( xml_name2 == XML_TRAFFIC_DATA_RX )
 					CSocket::m_Traffic.AddTraffic( ettDATARX, xml->content(n2).asULL() );
-				else if ( xml_name == XML_TRAFFIC_DATA_TX )
+				else if ( xml_name2 == XML_TRAFFIC_DATA_TX )
 					CSocket::m_Traffic.AddTraffic( ettDATATX, xml->content(n2).asULL() );
-				else if ( xml_name == XML_TRAFFIC_CONTROL_RX )
+				else if ( xml_name2 == XML_TRAFFIC_CONTROL_RX )
 					CSocket::m_Traffic.AddTraffic( ettCONTROLRX, xml->content(n2).asULL() );
-				else if ( xml_name == XML_TRAFFIC_CONTROL_TX )
+				else if ( xml_name2 == XML_TRAFFIC_CONTROL_TX )
 					CSocket::m_Traffic.AddTraffic( ettCONTROLTX, xml->content(n2).asULL() );
 			}
 		}
 		/* parse connection entrys */
-		else if ( xml->name(n1) == XML_CONNECTION )
+		else if ( xml_name1 == XML_CONNECTION )
 		{
 			for(n2=n1->xmlChildrenNode;n2!=0;n2=n2->next)
 			{
-				xml_name = xml->name(n2);
+				xml_name2 = xml->name(n2);
 
-				if ( xml_name == XML_MODE )
+				if ( xml_name2 == XML_MODE )
 				{
 					if ( xml->content(n2) == XML_MODE_PASSIVE )
 						eMode = ecmPASSIVE;
 					else
 						eMode = ecmACTIVE;
 				}
-				else if ( xml_name == XML_TCPLISTENPORT )
+				else if ( xml_name2 == XML_TCPLISTENPORT )
 					iTCPListenPort = xml->content(n2).asINT();
-				else if ( xml_name == XML_UDPLISTENPORT )
+				else if ( xml_name2 == XML_UDPLISTENPORT )
 					iUDPListenPort = xml->content(n2).asINT();
-				else if ( xml_name == XML_IP )
+				else if ( xml_name2 == XML_IP )
 					m_sHost = xml->content(n2);
-				else if ( xml_name == XML_LISTENHOST )
+				else if ( xml_name2 == XML_LISTENHOST )
 					m_sListenHost = xml->content(n2);
-				else if ( xml_name == XML_EXTERNALIP )
+				else if ( xml_name2 == XML_EXTERNALIP )
 					m_bExternalIP = xml->getBoolChild(n2);
-				else if ( xml_name == XML_SENDMESSAGEONACTIVEMODEREQUEST )
+				else if ( xml_name2 == XML_SENDMESSAGEONACTIVEMODEREQUEST )
 					bSendMessageOnActiveModeRequest = xml->getBoolChild(n2);
-				else if ( xml_name == XML_CHECKPRIVATEADDRESSSPACE )
+				else if ( xml_name2 == XML_CHECKPRIVATEADDRESSSPACE )
 					bCheckPrivateAddressSpace = xml->getBoolChild(n2);
 			}
 		}
 		/* other entrys */
-		else if ( xml->name(n1) == XML_OTHER )
+		else if ( xml_name1 == XML_OTHER )
 		{
 			for(n2=n1->xmlChildrenNode;n2!=0;n2=n2->next)
 			{
-				xml_name = xml->name(n2);
+				xml_name2 = xml->name(n2);
 
-				if ( xml_name == XML_HUBLISTSTORELOCAL )
+				if ( xml_name2 == XML_HUBLISTSTORELOCAL )
 					bHubListStoreLocal = xml->getBoolChild(n2);
-				else if ( xml_name == XML_RECONNECTCOUNT )
+				else if ( xml_name2 == XML_RECONNECTCOUNT )
 					iReconnectCount = xml->content(n2).asINT();
-				else if ( xml_name == XML_RECONNECTTIMEOUT )
+				else if ( xml_name2 == XML_RECONNECTTIMEOUT )
 					iReconnectTimeout = xml->content(n2).asINT();
-				else if ( xml_name == XML_TRANSFERRESPONSETIMEOUT )
+				else if ( xml_name2 == XML_TRANSFERRESPONSETIMEOUT )
 					iTransferResponseTimeout = xml->content(n2).asINT();
-				else if ( xml_name == XML_TRANSFERRESENDTIMEOUT )
+				else if ( xml_name2 == XML_TRANSFERRESENDTIMEOUT )
 					iTransferResendTimeout = xml->content(n2).asINT();
-				else if ( xml_name == XML_FORCEMOVEENABLED )
+				else if ( xml_name2 == XML_FORCEMOVEENABLED )
 					bForceMoveEnabled = xml->getBoolChild(n2);
-				else if ( xml_name == XML_DATADIR )
-					sDatadir = xml->content(n2);
-				else if ( xml_name == XML_CHATSENDOFFLINEMESSAGES )
+				else if ( xml_name2 == XML_DCLIB_DATAPATH )
+					m_sDCLibDataPath = xml->content(n2);
+				else if ( xml_name2 == XML_CHATSENDOFFLINEMESSAGES )
 					bChatSendOfflineMessages = xml->getBoolChild(n2);
-				else if ( xml_name == XML_CHATRECVOFFLINEMESSAGES )
+				else if ( xml_name2 == XML_CHATRECVOFFLINEMESSAGES )
 					bChatRecvOfflineMessages = xml->getBoolChild(n2);
-				else if ( xml_name == XML_RELOAD_HUBLIST_TIME )
+				else if ( xml_name2 == XML_RELOAD_HUBLIST_TIME )
 					m_nReloadHubListTime = xml->content(n2).asINT();
+				else if ( xml_name2 == XML_DISABLEHASHLIST )
+					m_bDisableHashList = xml->getBoolChild(n2);
 			}
 		}
 		/* parse server entrys */
-		else if ( xml->name(n1) == XML_HUBLISTURL )
+		else if ( xml_name1 == XML_HUBLISTURL )
 		{
 			for(n2=n1->xmlChildrenNode;n2!=0;n2=n2->next)
 			{
-				xml_name = xml->name(n2);
-
-				if ( xml_name == XML_URL )
+				if ( xml->name(n2) == XML_URL )
 				{
 					DCConfigHubListUrl * hublisturl = new DCConfigHubListUrl();
 
 					for(n3=n2->xmlChildrenNode;n3!=0;n3=n3->next)
 					{
-						if ( xml->name(n3) == XML_NAME )
+						xml_name2 = xml->name(n3);
+						if ( xml_name2 == XML_NAME )
 							hublisturl->sUrl = xml->content(n3);
-						else if ( xml->name(n3) == XML_ENABLED )
+						else if ( xml_name2 == XML_ENABLED )
 							hublisturl->bEnabled = xml->getBoolChild(n3);
 					}
 
-					if ( hublisturl->sUrl.ToUpper() != CString("http://axljab.homelinux.org:8080/PublicHubList.config.bz2").ToUpper() )
+					if ( hublisturl->sUrl.ToUpper() != "" )
 					{
 						pHubListUrlList.Add(hublisturl);
 					}
@@ -1491,13 +1498,11 @@ void CConfig::ParseDCLibConfig( xmlNodePtr node )
 			}
 		}
 		/* parse search history */
-		else if ( xml->name(n1) == XML_SEARCH_HISTORY )
+		else if ( xml_name1 == XML_SEARCH_HISTORY )
 		{
 			for(n2=n1->xmlChildrenNode;n2!=0;n2=n2->next)
 			{
-				xml_name = xml->name(n2);
-
-				if ( xml_name == XML_NAME )
+				if ( xml->name(n2) == XML_NAME )
 					pSearchHistory->Add( new CString(xml->content(n2)) );
 			}
 		}
@@ -1533,6 +1538,7 @@ CString CConfig::GetDescription( bool raw, CString hubname, CString hubhost )
 	CString s = "";
 	bool tag = FALSE;
 	bool prof = FALSE;
+	enum eClientMode mode;
 	
 	// check profile
 	if ( (hubname != "") || (hubhost != "") )
@@ -1543,6 +1549,8 @@ CString CConfig::GetDescription( bool raw, CString hubname, CString hubhost )
 		}
 	}
 
+	mode = GetMode();
+	
 	Thread.Lock();
 	
 	if ( (GetAwayMode() == euamAWAY) && (raw == FALSE) )
@@ -1587,7 +1595,7 @@ CString CConfig::GetDescription( bool raw, CString hubname, CString hubhost )
 		s += VERSION;
 		s += ",M:";
 
-		switch (eMode)
+		switch (mode)
 		{
 			case ecmACTIVE:
 				s += "A";
@@ -1819,7 +1827,7 @@ bool CConfig::RemovePublicHub( CString name, CString /*host*/, CString /*descrip
 }
 
 /** */
-bool CConfig::AddPublicHub( const CString name, const CString host, const CString description, const CString usercount )
+bool CConfig::AddPublicHub( const CString & name, const CString & host, const CString & description, const CString usercount )
 {
 	if ( (name == "") || (host == "") )
 	{
@@ -1939,7 +1947,7 @@ CString CConfig::GetTCPHostString( bool addport )
 {
 	CSocket Socket;
 	CString s = "";
-	unsigned int n;
+	unsigned int n = 0;
 	
 	// get current port from listenmanager
 	if ( CListenManager::Instance() )
@@ -1971,7 +1979,6 @@ CString CConfig::GetTCPHostString( bool addport )
 	
 	if ( (s != "") && (addport == TRUE) )
 	{
-			
 		if ( n != 0 )
 		{
 			s = s + ":" + CString().setNum(n);
@@ -2028,9 +2035,9 @@ CString CConfig::GetUDPHostString( bool addport )
 /**
  * check if we can resolve the ip in active mode
 */
-eClientMode CConfig::GetMode()
+eClientMode CConfig::GetMode( bool setting )
 {
-	if ( eMode == ecmACTIVE )
+	if ( !setting && (eMode == ecmACTIVE) )
 	{
 		if ( GetTCPHostString() == "" )
 		{
@@ -2216,6 +2223,7 @@ CString CConfig::GetNick( CString hubname, CString hubhost )
 	DCConfigHubItem * hubitem;
 	DCConfigHubProfile * profile;
 	CString * ps = 0;
+	CString s;
 
 	hubitem = 0;
 
@@ -2241,6 +2249,14 @@ CString CConfig::GetNick( CString hubname, CString hubhost )
 		}
 	}
 
+	s = CConnectionManager::Instance()->GetNick(hubname,hubhost);
+	
+	if ( s != "" )
+	{
+		Thread.UnLock();
+		return s;
+	}
+
 	Thread.UnLock();
 
 	return GetNick();
@@ -2562,12 +2578,12 @@ bool CConfig::SaveHubProfile()
 
 		node = xmlNewChild( doc->children, 0, (const xmlChar*)XML_PROFILE, 0 );
 
-		xmlNewChild( node, 0, (const xmlChar*)XML_NAME, (const xmlChar*) xml->ToUTF8(pConfigHubProfile->m_sName).Data());
-		xmlNewChild( node, 0, (const xmlChar*)XML_NICK, (const xmlChar*) xml->ToUTF8(pConfigHubProfile->m_sNick).Data());
-		xmlNewChild( node, 0, (const xmlChar*)XML_PASSWORD, (const xmlChar*) xml->ToUTF8(pConfigHubProfile->m_sPassword).Data());
-		xmlNewChild( node, 0, (const xmlChar*)XML_EMAIL, (const xmlChar*) xml->ToUTF8(pConfigHubProfile->m_sEMail).Data());
+		xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_NAME, pConfigHubProfile->m_sName);
+		xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_NICK, pConfigHubProfile->m_sNick);
+		xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_PASSWORD, pConfigHubProfile->m_sPassword);
+		xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_EMAIL, pConfigHubProfile->m_sEMail);
 		xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_EMAIL_ENABLED, pConfigHubProfile->m_bEMail );
-		xmlNewChild( node, 0, (const xmlChar*)XML_DESCRIPTION, (const xmlChar*) xml->ToUTF8(pConfigHubProfile->m_sComment).Data());	
+		xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_DESCRIPTION, pConfigHubProfile->m_sComment);	
 		xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_DESCRIPTION_ENABLED, pConfigHubProfile->m_bComment );
 		xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_AUTO_CONNECT, pConfigHubProfile->m_bAutoConnect );
 		xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_SSL, pConfigHubProfile->m_bSSL );
@@ -2576,7 +2592,7 @@ bool CConfig::SaveHubProfile()
 
 	s = sConfigPath + DCPROF_CONFIG;
 
-	if ( ! (xmlSaveFormatFile(s.Data(),doc,1) == -1) )
+	if ( ! (xmlSaveFormatFileEnc(s.Data(),doc,"utf-8",1) == -1) )
 	{
 		err = TRUE;
 	}
@@ -2593,7 +2609,7 @@ bool CConfig::SaveHubProfile()
 bool CConfig::LoadHubProfile()
 {
 	bool err = TRUE;
-	CString s;
+	CString s, xml_name;
 	DCConfigHubProfile * pConfigHubProfile;
 	CXml *xml;
 	xmlNodePtr n1,n2,node;
@@ -2618,25 +2634,26 @@ bool CConfig::LoadHubProfile()
 
 						for(n2=n1->xmlChildrenNode;n2!=0;n2=n2->next)
 						{
-							if ( xml->name(n2) == XML_NAME )
+							xml_name = xml->name(n2);
+							if ( xml_name == XML_NAME )
 								pConfigHubProfile->m_sName = xml->content(n2);
-							else if ( xml->name(n2) == XML_NICK )
+							else if ( xml_name == XML_NICK )
 								pConfigHubProfile->m_sNick = xml->content(n2);
-							else if ( xml->name(n2) == XML_PASSWORD )
+							else if ( xml_name == XML_PASSWORD )
 								pConfigHubProfile->m_sPassword = xml->content(n2);
-							else if ( xml->name(n2) == XML_EMAIL )
+							else if ( xml_name == XML_EMAIL )
 								pConfigHubProfile->m_sEMail = xml->content(n2);
-							else if ( xml->name(n2) == XML_DESCRIPTION )
+							else if ( xml_name == XML_DESCRIPTION )
 								pConfigHubProfile->m_sComment = xml->content(n2);
-							else if ( xml->name(n2) == XML_AUTO_CONNECT )
+							else if ( xml_name == XML_AUTO_CONNECT )
 								pConfigHubProfile->m_bAutoConnect = xml->getBoolChild(n2);
-							else if ( xml->name(n2) == XML_SSL )
+							else if ( xml_name == XML_SSL )
 								pConfigHubProfile->m_bSSL = xml->getBoolChild(n2);
-							else if ( xml->name(n2) == XML_DESCRIPTION_TAG )
+							else if ( xml_name == XML_DESCRIPTION_TAG )
 								pConfigHubProfile->m_bTag = xml->getBoolChild(n2);
-							else if ( xml->name(n2) == XML_DESCRIPTION_ENABLED )
+							else if ( xml_name == XML_DESCRIPTION_ENABLED )
 								pConfigHubProfile->m_bComment = xml->getBoolChild(n2);
-							else if ( xml->name(n2) == XML_EMAIL_ENABLED )
+							else if ( xml_name == XML_EMAIL_ENABLED )
 								pConfigHubProfile->m_bEMail = xml->getBoolChild(n2);
 						}
 
@@ -2794,6 +2811,7 @@ CString CConfig::AliasToPath( CString file )
 				if ( dir.getFileSize(s1) > 0 )
 				{
 					s += DIRSEPARATOR + sfile;
+					s = dir.SimplePath(s);
 					Thread.UnLock();
 					return s;
 				}
