@@ -2,7 +2,7 @@
                            cfilemanager.cpp  -  description
                              -------------------
     begin                : Sat Aug 03 2002
-    copyright            : (C) 2002-2003 by Mathias Kster
+    copyright            : (C) 2002-2005 by Mathias Kster
     email                : mathen@users.berlios.de
  ***************************************************************************/
 
@@ -39,7 +39,8 @@
 #include <dclib/core/cbz.h>
 #include <dclib/core/che3.h>
 #include <dclib/csearchindex.h>
-#include <dclib/hash/mhash_tiger.h>
+#include <dclib/hash/tigertree.h>
+#include <dclib/core/cbase32.h>
 
 #include "cfilemanager.h"
 
@@ -88,7 +89,10 @@ struct filetype FileType[] = {
 
 	// DOCUMENT 4
 	{ "CFG",   eftDOCUMENT },
+	{ "CONF",  eftDOCUMENT },
 	{ "CPP",   eftDOCUMENT },
+	{ "CSS",   eftDOCUMENT },
+	{ "C",     eftDOCUMENT },
 	{ "DIZ",   eftDOCUMENT },
 	{ "DOC",   eftDOCUMENT },
 	{ "H",     eftDOCUMENT },
@@ -102,8 +106,10 @@ struct filetype FileType[] = {
 	{ "PS",    eftDOCUMENT },
 	{ "PDF",   eftDOCUMENT },
 	{ "SHTML", eftDOCUMENT },
+	{ "SXW",   eftDOCUMENT },
 	{ "TXT",   eftDOCUMENT },
 	{ "RFT",   eftDOCUMENT },
+	{ "RDF",   eftDOCUMENT },
 	{ "XML",   eftDOCUMENT },
 
 	// APPL 5
@@ -114,6 +120,7 @@ struct filetype FileType[] = {
 	{ "EXE",  eftAPPLICATION },
 	{ "HQX",  eftAPPLICATION },
 	{ "JS",   eftAPPLICATION },
+	{ "SH",   eftAPPLICATION },
 	{ "SO",   eftAPPLICATION },
 	{ "SYS",  eftAPPLICATION },
 	{ "VXD",  eftAPPLICATION },
@@ -141,6 +148,7 @@ struct filetype FileType[] = {
 	{ "PCX",  eftPICTURE },
 	{ "TGA",  eftPICTURE },
 	{ "TIF",  eftPICTURE },
+	{ "XPM",  eftPICTURE },
 
 	// VIDEO 7
 	{ "AVI",   eftVIDEO },
@@ -171,11 +179,10 @@ CFileManager::CFileManager()
 	m_nFileBaseIndex  = 0;
 	m_pSearchIndex    = new CSearchIndex();
 	m_pShareList      = new CShareList();
+	m_pHashMemory     = 0;
 
 	m_pFileManagerInfo = new CFileManagerInfo();
-	m_pFileManagerInfo->m_bCreateShareList   = FALSE;
-	m_pFileManagerInfo->m_bCreateSearchIndex = FALSE;
-	m_pFileManagerInfo->m_nShareListPercent  = 100;
+	m_pFileManagerInfo->m_nProgress = 100;
 
 	InitFileTypeList();
 
@@ -195,6 +202,14 @@ CFileManager::CFileManager()
 			{
 				CreateShareList();
 			}
+			else
+			{
+				m_pShareList->CreateList(m_pSearchIndex);
+			}
+		}
+		else
+		{
+			m_pShareList->CreateList(m_pSearchIndex);
 		}
 	}
 
@@ -259,13 +274,19 @@ CFileManager::~CFileManager()
 		m_pSearchIndex = 0;
 	}
 
+	if ( m_pHashMemory )
+	{
+		delete m_pHashMemory;
+		m_pHashMemory = 0;
+	}
+
 	UnLock();
 }
 
 /** */
-int CFileManager::GetShareBuffer( eShareBufferType type, CByteArray * sharebuffer )
+int CFileManager::GetShareBuffer( eShareBufferType type, CByteArray * sharebuffer, bool decompress )
 {
-	return m_pShareList->GetShareBuffer( type, sharebuffer );
+	return m_pShareList->GetShareBuffer( type, sharebuffer, decompress );
 }
 
 /** */
@@ -326,27 +347,59 @@ eFileTypes CFileManager::GetFileType( CString file )
 /** */
 CStringList * CFileManager::Search( CString s )
 {
-	if ( !m_pFileManagerInfo ||
-	     !m_pSearchIndex ||
-	     (m_pFileManagerInfo->m_bCreateSearchIndex == TRUE) ||
-	     (m_pFileManagerInfo->m_bCreateShareList == TRUE) )
-	{
+	if ( !m_pFileManagerInfo || !m_pSearchIndex )
+		return 0;
+
+	if ( m_pFileManagerInfo->m_eFileManagerStatus != efmsNONE )
 		return 0;
-	}
 
 	return m_pSearchIndex->Search(s);
 }
 
+/** */
+CStringList * CFileManager::SearchHash( CString s )
+{
+	CString ts;
+	CByteArray dst;
+	CBase32 base32;
+	
+	if ( !m_pFileManagerInfo || !m_pSearchIndex )
+		return 0;
+
+	if ( m_pFileManagerInfo->m_eFileManagerStatus != efmsNONE )
+		return 0;
+
+	if ( s.Left(4) != "TTH:" )
+		return 0;
+
+	ts = s.Mid(4,s.Length()-4);
+	
+	if ( base32.Decode(&dst,&ts) != TIGERSIZE )
+		return 0;
+
+	return m_pSearchIndex->SearchHash(dst.Data());
+}
+
+/** */
+CString CFileManager::GetHash( ulonglong hbi )
+{
+	if ( !m_pFileManagerInfo || !m_pSearchIndex )
+		return "";
+
+	if ( m_pFileManagerInfo->m_eFileManagerStatus != efmsNONE )
+		return "";
+	
+	return m_pSearchIndex->GetHash(hbi);
+}
+
 /** */
 bool CFileManager::GetFileBaseObject( CString id, struct filebaseobject * fbo, CString & filename )
 {
-	if ( !m_pFileManagerInfo ||
-	     !m_pSearchIndex ||
-	     (m_pFileManagerInfo->m_bCreateSearchIndex == TRUE) ||
-	     (m_pFileManagerInfo->m_bCreateShareList == TRUE) )
-	{
+	if ( !m_pFileManagerInfo || !m_pSearchIndex )
+		return FALSE;
+
+	if ( m_pFileManagerInfo->m_eFileManagerStatus != efmsNONE )
 		return FALSE;
-	}
 
 	return m_pSearchIndex->GetFileBaseObject(id,fbo,filename);
 }
@@ -356,44 +409,50 @@ bool CFileManager::CreateShareList()
 {
 	bool err = FALSE;
 
+	if ( m_pFileManagerInfo->m_eFileManagerStatus != efmsNONE )
+		return err;
+	
 	Lock();
 
-	if ( (m_pFileManagerInfo->m_bCreateSearchIndex == FALSE) &&
-	     (m_pFileManagerInfo->m_bCreateShareList == FALSE) )
+	if ( Start() == -1 )
 	{
-		m_SharedFolders.Clear();
-
-		m_pShareFolder = 0;
-		m_nShareSize   = 0;
-		m_sShareBuffer = "";
-
-		m_pSearchIndex->Reset();
-
-		if ( CConfig::Instance()->GetSharedFolders(&m_SharedFolders) != 0 )
-		{
-			if ( m_pFileNameList )
-				delete m_pFileNameList;
-			m_pFileNameList  = new CStringList(25);
-
-			m_pFileManagerInfo->m_nShareListPercent = 0;
-			m_pFileManagerInfo->m_bCreateShareList  = TRUE;
-
-			if ( CDownloadManager::Instance() )
-				CDownloadManager::Instance()->SendFileManagerInfo(m_pFileManagerInfo);
-
-			err = TRUE;
-		}
-		else
-		{
-			m_pShareList->SetShareBuffer( "", 0 );
-
-			m_pSearchIndex->SaveIndex();
-		}
+		UnLock();
+		return err;
 	}
+	
+	m_SharedFolders.Clear();
 
-	if ( err == TRUE )
-	{	
-		Start();
+	m_pShareFolder = 0;
+	m_nShareSize   = 0;
+	m_sShareIndexBuffer = "";
+
+	m_pSearchIndex->Reset();
+
+	if ( CConfig::Instance()->GetSharedFolders(&m_SharedFolders) > 0 )
+	{
+		if ( m_pFileNameList )
+			delete m_pFileNameList;
+		m_pFileNameList  = new CStringList(25);
+
+		m_pFileManagerInfo->m_nProgress = 0;
+		m_pFileManagerInfo->m_eFileManagerStatus = efmsCREATESHARELIST;
+
+		if ( CDownloadManager::Instance() )
+			CDownloadManager::Instance()->SendFileManagerInfo(m_pFileManagerInfo);
+
+		err = TRUE;
+	}
+	else
+	{
+		m_pShareList->SetIndexBuffer( "" );
+		m_pShareList->CreateList(m_pSearchIndex);
+
+		m_pSearchIndex->SaveIndex();
+		
+		UnLock();
+		Stop();
+		
+		return err;
 	}
 
 	UnLock();
@@ -404,15 +463,14 @@ bool CFileManager::CreateShareList()
 /** */
 void CFileManager::CreateSearchIndex()
 {
-	if ( (m_pFileManagerInfo->m_bCreateSearchIndex == FALSE) &&
-	     (m_pFileManagerInfo->m_bCreateShareList == FALSE) )
+	if ( m_pFileManagerInfo->m_eFileManagerStatus == efmsIDLE )
 	{
 		m_nFileBaseIndex = 0;
 
 		m_pSearchIndex->ResetIndex();
 
-		m_pFileManagerInfo->m_nSearchIndexPercent = 0;
-		m_pFileManagerInfo->m_bCreateSearchIndex  = TRUE;
+		m_pFileManagerInfo->m_nProgress = 0;
+		m_pFileManagerInfo->m_eFileManagerStatus = efmsCREATESEARCHINDEX;
 
 		Start();
 	}
@@ -421,7 +479,21 @@ void CFileManager::CreateSearchIndex()
 /** */
 bool CFileManager::CreateHashList()
 {
-	return FALSE;
+	bool res = FALSE;
+
+	if ( m_pFileManagerInfo->m_eFileManagerStatus == efmsIDLE )
+	{
+		m_nFileBaseIndex = 0;
+
+		m_pFileManagerInfo->m_nProgress = 0;
+		m_pFileManagerInfo->m_eFileManagerStatus = efmsCREATEHASHLIST;
+
+		Start();
+		
+		res = TRUE;
+	}
+
+	return res;
 }
 
 /** */
@@ -477,13 +549,13 @@ void CFileManager::ThreadCreateSearchIndex()
 
 			m_pSearchIndex->SaveIndex();
 
-			m_pFileManagerInfo->m_nSearchIndexPercent = 100;
-			m_pFileManagerInfo->m_bCreateSearchIndex  = FALSE;
+			m_pFileManagerInfo->m_nProgress = 100;
+			m_pFileManagerInfo->m_eFileManagerStatus = efmsIDLE;
 
 			if ( CDownloadManager::Instance() )
 				CDownloadManager::Instance()->SendFileManagerInfo(m_pFileManagerInfo);
 
-			Stop();
+			CreateHashList();
 
 			return;
 		}
@@ -493,9 +565,9 @@ void CFileManager::ThreadCreateSearchIndex()
 	{
 		percent = (m_nFileBaseIndex*100.0)/m_pSearchIndex->IndexCount();
 
-		if ( m_pFileManagerInfo->m_nSearchIndexPercent != percent )
+		if ( m_pFileManagerInfo->m_nProgress != percent )
 		{
-			m_pFileManagerInfo->m_nSearchIndexPercent = percent;
+			m_pFileManagerInfo->m_nProgress = percent;
 
 			if ( CDownloadManager::Instance() )
 				CDownloadManager::Instance()->SendFileManagerInfo(m_pFileManagerInfo);
@@ -534,10 +606,11 @@ void CFileManager::ThreadCreateShareList()
 	}
 	else
 	{
-		m_pFileManagerInfo->m_nShareListPercent = 100;
+		m_pFileManagerInfo->m_nProgress = 100;
 		printf("ready create sharelist\n");
 
-		m_pShareList->SetShareBuffer( m_sShareBuffer, m_nShareSize );
+		m_pShareList->SetIndexBuffer( m_sShareIndexBuffer );
+		m_pShareList->CreateList(m_pSearchIndex);
 
 		if ( m_pFileNameList )
 		{
@@ -545,7 +618,7 @@ void CFileManager::ThreadCreateShareList()
 			m_pFileNameList = 0;
 		}
 
-		m_pFileManagerInfo->m_bCreateShareList = FALSE;
+		m_pFileManagerInfo->m_eFileManagerStatus = efmsIDLE;
 
 		if ( CDownloadManager::Instance() )
 			CDownloadManager::Instance()->SendFileManagerInfo(m_pFileManagerInfo);
@@ -566,6 +639,10 @@ void CFileManager::ThreadCreateShareList( int depth, CString curr, CString rel,
 	CString path;
 	CFileInfo *fileinfo;
 	int x,z;
+	unsigned long index;
+
+	if ( Stopped() )
+		return;
 
 	path = dir.SimplePath(m_pShareFolder->m_sPath+DIRSEPARATOR+curr+DIRSEPARATOR+rel);
 
@@ -581,9 +658,9 @@ void CFileManager::ThreadCreateShareList( int depth, CString curr, CString rel,
 	z++;
 	// use alias
 	if ( depth == 0 )
-		m_sShareBuffer += m_pShareFolder->m_sAlias + "\xd\xa";
+		m_sShareIndexBuffer += m_pShareFolder->m_sAlias + "\xd\xa";
 	else
-		m_sShareBuffer += d + rel + "\xd\xa";
+		m_sShareIndexBuffer += d + rel + "\xd\xa";
 	d += "\x9";
 
 	// get all files
@@ -593,6 +670,9 @@ void CFileManager::ThreadCreateShareList( int depth, CString curr, CString rel,
 		fileinfo = 0;
 		while( (fileinfo=list.Next(fileinfo)) != 0 )
 		{
+			if ( Stopped() )
+				return;
+
 			if ( fileinfo->name != "" )
 			{
 				if ( fileinfo->m_bSymlink )
@@ -609,12 +689,13 @@ void CFileManager::ThreadCreateShareList( int depth, CString curr, CString rel,
 
 						m_pFileNameList->Add(s,new CString());
 						m_nShareSize   += fileinfo->size;
-						m_sShareBuffer += d + fileinfo->name + "|" + CString().setNum(fileinfo->size) + "\xd\xa";
+						
+						// add file to the file manager ...
+						index = m_pSearchIndex->AddIndex( fileinfo, dir.SimplePath(m_pShareFolder->m_sAlias+DIRSEPARATOR+curr+DIRSEPARATOR+rel), GetFileType(fileinfo->name) );
+						
+						m_sShareIndexBuffer += d + fileinfo->name + "|" + CString().setNum(index) + "\xd\xa";
 					}
 				}
-
-				// add file to the file manager ...
-				m_pSearchIndex->AddIndex( fileinfo, dir.SimplePath(m_pShareFolder->m_sAlias+DIRSEPARATOR+curr+DIRSEPARATOR+rel), GetFileType(fileinfo->name) );
 			}
 		}
 	}
@@ -625,12 +706,15 @@ void CFileManager::ThreadCreateShareList( int depth, CString curr, CString rel,
 		if ( list.Count() > 0 )
 			percent /= list.Count();
 		else
-			m_pFileManagerInfo->m_nShareListPercent += percent;
+			m_pFileManagerInfo->m_nProgress += percent;
 
 		// add dir
 		fileinfo = 0;
 		while( (fileinfo=list.Next(fileinfo)) != 0 )
 		{
+			if ( Stopped() )
+				return;
+
 			if ( (fileinfo->name != ".") && (fileinfo->name != "..") )
 			{
 				if ( fileinfo->m_bSymlink )
@@ -649,12 +733,12 @@ void CFileManager::ThreadCreateShareList( int depth, CString curr, CString rel,
 						ThreadCreateShareList(z,curr+DIRSEPARATOR+rel,fileinfo->name,percent);
 					}
 					else
-						m_pFileManagerInfo->m_nShareListPercent += percent;
+						m_pFileManagerInfo->m_nProgress += percent;
 
 				}
 			}
 			else
-				m_pFileManagerInfo->m_nShareListPercent += percent;
+				m_pFileManagerInfo->m_nProgress += percent;
 
 			if ( CDownloadManager::Instance() )
 				CDownloadManager::Instance()->SendFileManagerInfo(m_pFileManagerInfo);
@@ -790,6 +874,115 @@ ulonglong CFileManager::CalcShareSize( int depth, CString base, CString curr, CS
 /** */
 void CFileManager::ThreadCreateHashList()
 {
+	CString s,pa;
+	struct filebaseobject fbo;
+	ulonglong hbi;
+	struct tt_context tth;
+	unsigned char hash[TIGERSIZE];
+	int len;
+	CFile f;
+	CBase32 base32;
+	double percent;
+
+	if ( !m_pHashMemory )
+	{
+		m_pHashMemory = new CByteArray();
+		m_pHashMemory->SetSize(1024*1024);
+	}
+	
+	if ( m_pSearchIndex->GetFileBaseObject(CString().setNum(m_nFileBaseIndex),&fbo,s) == TRUE )
+	{
+		pa = CConfig::Instance()->AliasToPath(s);
+		
+		if ( pa == "" )
+		{
+			m_nFileBaseIndex++;
+			return;
+		}
+		
+		// check if we have the hash
+		if ( m_pSearchIndex->FindHashBaseIndex(&fbo,&hbi) )
+		{
+			if ( dclibVerbose() > 0 )
+				printf("hash found, no changes in file\n");
+			fbo.m_nHashIndex = hbi;
+			m_pSearchIndex->UpdateIndex(m_nFileBaseIndex,&fbo);
+		}
+		else
+		{
+			if ( CConfig::Instance()->GetDisableHashList() == FALSE )
+			{
+				if ( f.Open( pa, IO_RAW | IO_READONLY ) == TRUE )
+				{
+					tt_init(&tth);
+
+					while ( (len = f.Read( (char*)m_pHashMemory->Data(), 1024*1024)) > 0 )
+					{
+						tt_update(&tth,m_pHashMemory->Data(),len);
+				
+						if ( Stopped() )
+							break;
+					}
+					
+					f.Close();
+			
+					if ( !Stopped() )
+					{
+						tt_digest(&tth,(unsigned char*)hash);
+						m_pSearchIndex->AddHashIndex(m_nFileBaseIndex,hash);
+					}
+					else
+					{
+						printf("create hash stop\n");
+						m_pFileManagerInfo->m_eFileManagerStatus = efmsIDLE;
+					}
+				}
+			}
+		}
+		
+		m_nFileBaseIndex++;
+	
+		if ( m_pSearchIndex->IndexCount() > 0 )
+		{
+			percent = (m_nFileBaseIndex*100.0)/m_pSearchIndex->IndexCount();
+
+			if ( (percent-m_pFileManagerInfo->m_nProgress) > 0.5 )
+			{
+				m_pFileManagerInfo->m_nProgress = percent;
+
+				if ( CDownloadManager::Instance() )
+					CDownloadManager::Instance()->SendFileManagerInfo(m_pFileManagerInfo);
+			}
+		}
+	}
+	else
+	{
+		m_pFileManagerInfo->m_eFileManagerStatus = efmsIDLE;
+	}
+	
+	if ( m_pFileManagerInfo->m_eFileManagerStatus == efmsIDLE )
+	{
+		printf("create hash end\n");
+		m_pFileManagerInfo->m_nProgress = 100;
+
+		if ( CDownloadManager::Instance() )
+			CDownloadManager::Instance()->SendFileManagerInfo(m_pFileManagerInfo);
+
+		if ( m_pHashMemory )
+		{
+			delete m_pHashMemory;
+			m_pHashMemory = 0;
+		}
+
+		m_pSearchIndex->SaveIndex();
+		m_pShareList->CreateList(m_pSearchIndex);
+		
+		m_pFileManagerInfo->m_eFileManagerStatus = efmsNONE;
+		if ( CDownloadManager::Instance() )
+			CDownloadManager::Instance()->SendFileManagerInfo(m_pFileManagerInfo);
+
+		Stop();
+	}
 }
 
 /** */
@@ -799,15 +992,21 @@ void CFileManager::Thread( CObject * )
 
 	Lock();
 
-	if ( m_pFileManagerInfo->m_bCreateShareList == TRUE )
+	switch(m_pFileManagerInfo->m_eFileManagerStatus)
 	{
-		ThreadCreateShareList();
-	}
-	else if ( m_pFileManagerInfo->m_bCreateSearchIndex == TRUE )
-	{
-		ThreadCreateSearchIndex();
-
-		wait = 0;
+		case efmsCREATESHARELIST:
+			ThreadCreateShareList();
+			break;
+		case efmsCREATESEARCHINDEX:
+			ThreadCreateSearchIndex();
+			wait = 0;
+			break;
+		case efmsCREATEHASHLIST:
+			ThreadCreateHashList();
+			wait = 0;
+			break;
+		default:
+			break;
 	}
 
 	UnLock();
