@@ -2,7 +2,7 @@
                           cmessagehandler.cpp  -  description
                              -------------------
     begin                : Sun Sep 30 2001
-    copyright            : (C) 2001-2003 by Mathias Kster
+    copyright            : (C) 2001-2005 by Mathias Kster
     email                : mathen@users.berlios.de
  ***************************************************************************/
 
@@ -23,6 +23,8 @@
 #include <dclib/dcobject.h>
 #include <dclib/core/cbytearray.h>
 #include <dclib/core/cbase64.h>
+#include <dclib/core/cxml.h>
+#include <dclib/core/csocket.h>
 
 #include "cmessagehandler.h"
 
@@ -75,6 +77,12 @@
 #define DC_S_YOUR_IP			"$YourIP "
 #define DC_S_USER_COMMAND		"$UserCommand " // ??? ( $UserCommand 0 / $UserCommand 1 Read Help$<%[mynick]> +help&#124;<%[mynick]> -help&#124; )
 
+#define DC_S_MESSAGE_UGETBLOCK		"$UGetBlock "
+// ZLIB compression messages
+#define DC_S_MESSAGE_GETZBLOCK		"$GetZBlock "
+#define DC_S_MESSAGE_SENDING		"$Sending "
+#define DC_S_MESSAGE_FAILED		"$Failed "
+
 CMessageHandler::CMessageHandler()
 {
 }
@@ -255,6 +263,12 @@ eDCMessage CMessageHandler::Parse( const CString * sMessage, int & pointer, CObj
 
 			return DC_MESSAGE_GET;
 		}
+		else if ( GetContent(DC_S_MESSAGE_UGETBLOCK,&t,sContent) != -1 )
+		{
+			Object = ParseUGetBlock(sContent);
+
+			return DC_MESSAGE_GET;
+		}
 		else if ( GetContent(DC_S_MESSAGE_ERROR,&t,sContent) != -1 )
 		{
 			Object = ParseError(sContent);
@@ -285,6 +299,12 @@ eDCMessage CMessageHandler::Parse( const CString * sMessage, int & pointer, CObj
 
 			return DC_MESSAGE_CANCELED;
 		}
+		else if ( GetContent(DC_S_MESSAGE_SENDING,&t,sContent) != -1 )
+		{
+			Object = ParseSending(sContent);
+
+			return DC_MESSAGE_SENDING;
+		}
 		else if ( GetContent(DC_S_MESSAGE_SEND,&t,sContent) != -1 )
 		{
 			Object = new CMessageSend();
@@ -363,6 +383,24 @@ eDCMessage CMessageHandler::Parse( const CString * sMessage, int & pointer, CObj
 
 			return DC_MESSAGE_USER_COMMAND;
 		}
+		else if ( GetContent(DC_S_YOUR_IP,&t,sContent) != -1 )
+		{
+			Object = ParseYourIP(sContent);
+
+			return DC_MESSAGE_YOURIP;
+		}
+		else if ( GetContent(DC_S_MESSAGE_GETZBLOCK,&t,sContent) != -1 )
+		{
+			Object = ParseGetZBlock(sContent);
+
+			return DC_MESSAGE_GETZBLOCK;
+		}
+		else if ( GetContent(DC_S_MESSAGE_FAILED,&t,sContent) != -1 )
+		{
+			Object = ParseError(sContent);
+
+			return DC_MESSAGE_ERROR;
+		}
 		else
 		{
 			printf("!!!!!!!!!!!!!! unknown start ");
@@ -631,6 +669,8 @@ CObject * CMessageHandler::ParseMyInfo( CString sContent )
 			msg->m_eUserSpeed = eus288KBPS;
 		else if ( s == "56Kbps" )
 			msg->m_eUserSpeed = eus56KBPS;
+		else if ( s == "Modem" )
+			msg->m_eUserSpeed = eusMODEM;
 		else if ( s == "ISDN" )
 			msg->m_eUserSpeed = eusISDN;
 		else if ( s == "DSL" )
@@ -682,6 +722,12 @@ CObject * CMessageHandler::ParseMyInfo( CString sContent )
 			msg->m_eClientVersion = eucvDCPRO;
 		else if ( (i=s.FindRev("<iDC")) != -1 )
 			msg->m_eClientVersion = eucvIDC;
+		else if ( (i=s.FindRev("<PerlDC")) != -1 )
+			msg->m_eClientVersion = eucvPERLDC;
+		else if ( (i=s.FindRev("<")) != -1 )
+			msg->m_eClientVersion = eucvUNKNOWN;
+		else
+			msg->m_eClientVersion = eucvNONE;
 
 		msg->m_eClientMode = ecmPASSIVE;
 
@@ -691,18 +737,36 @@ CObject * CMessageHandler::ParseMyInfo( CString sContent )
 			msg->m_sComment    = s.Left(i);
 			msg->m_sVerComment = s.Mid(i,i1-i+1);
 
-			// search for the active tag
-			// A: active, P: passive 5: socks5
-			if ( msg->m_sVerComment.Find("M:A") != -1 )
+			// try to find some default tags for unknown clients
+			if ( msg->m_eClientVersion == eucvUNKNOWN )
 			{
-				msg->m_eClientMode = ecmACTIVE;
+				if ( (msg->m_sVerComment.Find("V:") == -1) ||
+				     (msg->m_sVerComment.Find("M:") == -1) )
+				{
+					msg->m_eClientVersion = eucvNONE;
+				}
+			}
+			
+			if ( msg->m_eClientVersion != eucvNONE )
+			{
+				// search for the active tag
+				// A: active, P: passive 5: socks5
+				if ( msg->m_sVerComment.Find("M:A") != -1 )
+				{
+					msg->m_eClientMode = ecmACTIVE;
+				}
 			}
 		}
-		else
+		
+		// reset values
+		if ( msg->m_eClientVersion == eucvNONE )
 		{
-			msg->m_sComment       = s;
-			msg->m_eClientVersion = eucvNONE;
+			msg->m_sComment    = s;
+			msg->m_sVerComment = "";
 		}
+		
+		// set myinfo valid flag
+		msg->m_bValid = TRUE;
 	}
 
 	return msg;
@@ -740,6 +804,10 @@ CObject * CMessageHandler::ParseConnectToMe( CString sContent )
 		else
 		{
 			msg->m_nPort = s.asINT();
+			
+			// fix negative ports
+			if ( msg->m_nPort < 0 )
+				msg->m_nPort += 65536;
 		}
 	}
 
@@ -892,6 +960,10 @@ CObject * CMessageHandler::ParseSearch( CString sContent )
 			{
 				msg->m_sSource = s.Mid(0,i6);
 				msg->m_nPort   = s1.asINT();
+				
+				// fix negative ports
+				if ( msg->m_nPort < 0 )
+					msg->m_nPort += 65536;
 			}
 		}
 
@@ -903,9 +975,9 @@ CObject * CMessageHandler::ParseSearch( CString sContent )
 
 		// at most or at least
 		if ( sContent.Mid(i1+1,i2-i1-1) == "F" )
-			msg->m_bSizeAtMost = FALSE;
+			msg->m_eSizeType = esstATLEAST;
 		else
-			msg->m_bSizeAtMost = TRUE;
+			msg->m_eSizeType = esstATMOST;
 
 		// size
 		s1 = sContent.Mid(i2+1,i3-i2-1);
@@ -941,6 +1013,9 @@ CObject * CMessageHandler::ParseSearch( CString sContent )
 			case 8:
 				msg->m_eFileType = eftFOLDER;
 				break;
+			case 9:
+				msg->m_eFileType = eftHASH;
+				break;
 			default:
 				msg->m_eFileType = eftUNKNOWN;
 				break;
@@ -1047,6 +1122,17 @@ CObject * CMessageHandler::ParseSearchResult( CString sContent )
 			}
 		}
 
+		// check negative ports
+		CString ip;
+		unsigned int port;
+		
+		CSocket::ParseHost( msg->m_sHubHost, ip, port );
+		
+		msg->m_sHubHost = ip;
+		
+		if ( port != 0 )
+			 msg->m_sHubHost += ":" + CString().setNum(port);
+
 		// parse special content in hubname
 		if ( msg->m_sHubName.Left(4) == "TTH:" )
 		{
@@ -1164,6 +1250,10 @@ CObject * CMessageHandler::ParseForceMove( CString sContent )
 		{
 			msg->m_sHost = sContent.Mid(0,i);
 			msg->m_nPort = s.asINT();
+			
+			// fix negative ports
+			if ( msg->m_nPort < 0 )
+				msg->m_nPort += 65536;
 		}
 	}
 
@@ -1250,6 +1340,36 @@ CObject * CMessageHandler::ParseGet( CString sContent )
 	return msg;
 }
 
+/** */
+CObject * CMessageHandler::ParseUGetBlock( CString sContent )
+{
+	int i,i1;
+	CString s;
+	CMessageGet * msg=0;
+
+	if ( (msg = new CMessageGet()) != 0 )
+	{
+		i = sContent.Find(' ');
+		i1= sContent.Find(' ',i+1);
+		msg->m_nPos  = sContent.Mid(0,i).asULL();
+		msg->m_nSize = sContent.Mid(i+1,i1-i).asULL();
+		s = sContent.Mid(i1+1,sContent.Length()-i1-1);
+		msg->m_sFilename = CXml().FromUtf8(s);
+		
+		// fix start position
+		msg->m_nPos++;
+/*
+		printf("UGET '%s'\n%llu %llu\n'%s'\n",
+			sContent.Data(),
+			msg->m_nPos,
+			msg->m_nSize,
+			msg->m_sFilename.Data());
+*/
+	}
+
+	return msg;
+}
+
 /** */
 CObject * CMessageHandler::ParseError( CString sContent )
 {
@@ -1315,6 +1435,32 @@ CObject * CMessageHandler::ParseLogedIn( CString sContent )
 	return msg;
 }
 
+#define DC_SUPPORT_TAG_BZLIST		"BZList "
+#define DC_SUPPORT_TAG_SSL		"SSL "
+#define DC_SUPPORT_TAG_CHUNK		"CHUNK "
+#define DC_SUPPORT_TAG_MINISLOTS	"MiniSlots "
+#define DC_SUPPORT_TAG_XMLBZLIST	"XmlBZList "
+#define DC_SUPPORT_TAG_GETTESTZBLOCK	"GetTestZBlock "
+#define DC_SUPPORT_TAG_GETZBLOCK	"GetZBlock "
+#define DC_SUPPORT_TAG_ADCGET		"ADCGet "
+#define DC_SUPPORT_TAG_TTHL		"TTHL "
+#define DC_SUPPORT_TAG_TTHF		"TTHF "
+#define DC_SUPPORT_TAG_ZLIG		"ZLIG "
+#define DC_SUPPORT_TAG_USER_CMD		"UserCommand "
+#define DC_SUPPORT_TAG_NOGETINFO	"NoGetINFO "
+#define DC_SUPPORT_TAG_USERIP		"UserIP "
+#define DC_SUPPORT_TAG_USERIP2		"UserIP2 "
+#define DC_SUPPORT_TAG_XRULES		"XRules "
+#define DC_SUPPORT_TAG_BOTINFO		"BotINFO "
+#define DC_SUPPORT_TAG_BOTINFO_2	"BotInfo "
+#define DC_SUPPORT_TAG_NOHELLO		"NoHello "
+#define DC_SUPPORT_TAG_OPPLUS		"OpPlus "
+#define DC_SUPPORT_TAG_MCTO		"MCTo "
+#define DC_SUPPORT_TAG_BOTLIST		"BotList "
+#define DC_SUPPORT_TAG_CLIENTID		"ClientID "
+#define DC_SUPPORT_TAG_SECUREDEXECUTOR	"SecuredExecutor "
+#define DC_SUPPORT_TAG_FEED		"Feed "
+
 /** */
 CObject * CMessageHandler::ParseSupports( CString sContent )
 {
@@ -1324,6 +1470,8 @@ CObject * CMessageHandler::ParseSupports( CString sContent )
 
 	if ( (msg = new CMessageSupports()) != 0 )
 	{
+		msg->m_sContent = sContent;
+
 		i = i1 = 0;
 
 		while( (i1=sContent.Find(' ',i)) != -1 )
@@ -1332,49 +1480,105 @@ CObject * CMessageHandler::ParseSupports( CString sContent )
 
 			s = sContent.Mid(i,i1-i);
 
-			if ( s == "BZList " )
+			if ( s == DC_SUPPORT_TAG_BZLIST )
 			{
 				msg->m_bBZList = TRUE;
 			}
-			else if ( s == "SSL " )
+			else if ( s == DC_SUPPORT_TAG_SSL )
 			{
 				msg->m_bSSL = TRUE;
 			}
-			else if ( s == "CHUNK " )
+			else if ( s == DC_SUPPORT_TAG_CHUNK )
 			{
 				msg->m_bChunk = TRUE;
 			}
-			else if ( s == "MiniSlots " )
+			else if ( s == DC_SUPPORT_TAG_MINISLOTS )
 			{
 				msg->m_bMiniSlots = TRUE;
 			}			
-			else if ( s == "XmlBZList " )
+			else if ( s == DC_SUPPORT_TAG_XMLBZLIST )
 			{
 				msg->m_bXMLBZList = TRUE;
 			}
-			else if ( s == "GetTestZBlock " )
+			else if ( s == DC_SUPPORT_TAG_GETTESTZBLOCK )
 			{
 				// nothing ...
 			}
-			else if ( s == "GetZBlock " )
+			else if ( s == DC_SUPPORT_TAG_GETZBLOCK )
 			{
 				msg->m_bZBlock = TRUE;
 			}
-			else if ( s == "UserCommand " )
+			else if ( s == DC_SUPPORT_TAG_ADCGET )
 			{
-				// TODO
+				msg->m_bADCGet = TRUE;
 			}
-			else if ( s == "NoGetINFO " )
+			else if ( s == DC_SUPPORT_TAG_TTHL )
 			{
-				// TODO
+				msg->m_bTTHL = TRUE;
 			}
-			else if ( s == "UserIP2 " )
+			else if ( s == DC_SUPPORT_TAG_TTHF )
 			{
-				// TODO
+				msg->m_bTTHF = TRUE;
 			}
-			else if ( s == "XRules " )
+			else if ( s == DC_SUPPORT_TAG_ZLIG )
 			{
-				// TODO
+				msg->m_bZLIG = TRUE;
+			}
+			else if ( s == DC_SUPPORT_TAG_USER_CMD )
+			{
+				msg->m_bUserCmd = TRUE;
+			}
+			else if ( s == DC_SUPPORT_TAG_NOGETINFO )
+			{
+				msg->m_bNoGetInfo = TRUE;
+			}
+			else if ( s == DC_SUPPORT_TAG_USERIP )
+			{
+				msg->m_bUserIP = TRUE;
+			}
+			else if ( s == DC_SUPPORT_TAG_USERIP2 )
+			{
+				msg->m_bUserIP2 = TRUE;
+			}
+			else if ( s == DC_SUPPORT_TAG_XRULES )
+			{
+				msg->m_bXRules = TRUE;
+			}
+			else if ( s == DC_SUPPORT_TAG_BOTINFO )
+			{
+				msg->m_bBotINFO = TRUE;
+			}
+			else if ( s == DC_SUPPORT_TAG_BOTINFO_2 )
+			{
+				msg->m_bBotINFO = TRUE;
+			}
+			else if ( s == DC_SUPPORT_TAG_NOHELLO )
+			{
+				msg->m_bNoHello = TRUE;
+			}
+			else if ( s == DC_SUPPORT_TAG_OPPLUS )
+			{
+				msg->m_bOpPlus = TRUE;
+			}
+			else if ( s == DC_SUPPORT_TAG_MCTO )
+			{
+				msg->m_bMCTo = TRUE;
+			}
+			else if ( s == DC_SUPPORT_TAG_BOTLIST )
+			{
+				msg->m_bBotList = TRUE;
+			}
+			else if ( s == DC_SUPPORT_TAG_CLIENTID )
+			{
+				msg->m_bClientID = TRUE;
+			}
+			else if ( s == DC_SUPPORT_TAG_SECUREDEXECUTOR )
+			{
+				msg->m_bSecExec = TRUE;
+			}
+			else if ( s == DC_SUPPORT_TAG_FEED )
+			{
+				msg->m_bFeed = TRUE;
 			}
 			else
 			{
@@ -1444,3 +1648,31 @@ CObject * CMessageHandler::ParseUserCommand( CString sContent )
 {
 	return 0;
 }
+
+/** */
+CObject * CMessageHandler::ParseYourIP( CString sContent )
+{
+	return 0;
+}
+
+/** */
+CObject * CMessageHandler::ParseGetZBlock( CString sContent )
+{
+	return 0;
+}
+
+/** */
+CObject * CMessageHandler::ParseSending( CString sContent )
+{
+	CMessageSending * msg=0;
+
+	if ( (msg = new CMessageSending()) != 0 )
+	{
+		if ( "" != sContent )
+			msg->m_nLength = sContent.asULL();
+		else
+			msg->m_nLength = 0;
+	}
+
+	return msg;
+}
