@@ -250,14 +250,17 @@ void CSocket::ParseHost( CString host, CString & ip, unsigned int & port )
 		ip = host.Mid(i+1,host.Length()-i-1);
 	}
 
+	port = 0;
+
 	if ( (i < 0) || (ip == "") )
 	{
-		port = 0;
 		ip   = host;
 	}
 	else
 	{
-		port = ip.asINT();
+		if ( ip.asINT() < 0 )
+			port += 65536;
+		port += ip.asINT();
 		ip   = host.Mid(0,i);
 	}	
 }
