@@ -46,6 +46,7 @@
 #include <dclib/core/cbytearray.h>
 #include <dclib/cconfig.h>
 #include <dclib/core/platform.h>
+#include <dclib/cfilemanager.h>
 
 #include "ctransfer.h"
 
@@ -129,8 +130,7 @@ int CTransfer::CallBack_SendObject( CObject * Object )
 
 	if ( err == -1 )
 	{
-		if ( dclibVerbose() > 1 )
-			printf("CallBack failed (state)...\n");
+		DPRINTF("CallBack failed (state)...\n");
 		delete Object;
 	}
 
@@ -303,7 +303,7 @@ ulonglong CTransfer::GetTraffic()
 	}
 	else
 	{
-		printf("time is null\n"); fflush(stdout);
+		DPRINTF("time is null\n"); fflush(stdout);
 	}
 
 	return n;
@@ -330,8 +330,7 @@ void CTransfer::ConnectionState( eConnectionState state )
 	}
 	else if ( state == estSSLCONNECTED )
 	{
-		if ( dclibVerbose() > 0 )
-			printf("change to ssl mode success\n");
+		DPRINTF("change to ssl mode success\n");
 
 		CallBack_SendObject(new CMessageKey());
 	}
@@ -348,19 +347,19 @@ int CTransfer::StartDownload( CString dstfile, ulonglong startposition, ulonglon
 {
 	if ( (srcfile == "" ) && (eMedium == eltFILE) )
 	{
-		printf("ctransfer: wrong mode (empty file) %d\n",eMedium);
+		DPRINTF("ctransfer: wrong mode (empty file) %d\n",eMedium);
 		return -1;
 	}
 
 	if ( m_bIdle == FALSE )
 	{
-		printf("ctransfer: other transfer is running\n");
+		DPRINTF("ctransfer: other transfer is running\n");
 		return -1;
 	}
 
 	if ( eMode == estTRANSFERUPLOAD )
 	{
-		printf("ctransfer: wrong transfer mode\n");
+		DPRINTF("ctransfer: wrong transfer mode\n");
 		return -1;
 	}
 
@@ -382,25 +381,40 @@ int CTransfer::StartDownload( CString dstfile, ulonglong startposition, ulonglon
 	}
 	else
 	{
-#ifdef HAVE_LIBBZ2
 		if ( (GetDstFilename() == DC_USER_FILELIST_HE3) && (m_MessageSupports.m_bXMLBZList == TRUE) )
 		{
-			SendGet( DC_USER_FILELIST_XML, startposition+1 );
+			SendGet( DC_USER_FILELIST_XMLBZ, startposition+1 );
 		}
 		else if ( (GetDstFilename() == DC_USER_FILELIST_HE3) && (m_MessageSupports.m_bBZList == TRUE) )
 		{
 			SendGet( DC_USER_FILELIST_BZ, startposition+1 );
 		}
 		else
-#endif
 		{
-			if ( (m_MessageSupports.m_bChunk == FALSE) || (chunksize == 0) )
+			if ( chunksize == 0 )
 			{
 				SendGet( GetDstFilename(), startposition+1 );
+				m_nDataType = 0;
+			}
+			else if ( (m_MessageSupports.m_bChunk == FALSE) &&
+			          (m_MessageSupports.m_bZBlock == FALSE) )
+			{
+				SendGet( GetDstFilename(), startposition+1 );
+				m_nDataType = 0;
+			}
+			else if ( m_MessageSupports.m_bChunk == TRUE )
+			{
+				SendGet( GetDstFilename(), startposition+1, chunksize );
+				m_nDataType = 0;
+			}
+			else if ( m_MessageSupports.m_bZBlock == TRUE )
+			{
+				SendGetZBlock( GetDstFilename(), startposition, chunksize );
+				m_nDataType = 1;
 			}
 			else
 			{
-				SendGet( GetDstFilename(), startposition+1, chunksize );
+				return -1;
 			}
 		}
 	}
@@ -413,19 +427,19 @@ int CTransfer::StartUpload( CString dstfile, ulonglong length, ulonglong pos, ul
 {
 	if ( (srcfile == "" ) && (eMedium != eltBUFFER) )
 	{
-		printf("ctransfer: wrong mode %d\n",eMedium);
+		DPRINTF("ctransfer: wrong mode %d\n",eMedium);
 		return -1;
 	}
 
 	if ( m_bIdle == FALSE )
 	{
-		printf("ctransfer: other transfer is running\n");
+		DPRINTF("ctransfer: other transfer is running\n");
 		return -1;
 	}
 
 	if ( eMode == estTRANSFERDOWNLOAD )
 	{
-		printf("ctransfer: wrong transfer mode\n");
+		DPRINTF("ctransfer: wrong transfer mode\n");
 		return -1;
 	}
 
@@ -452,13 +466,90 @@ int CTransfer::StartUpload( CString dstfile, ulonglong length, ulonglong pos, ul
 	return 0;
 }
 
+/** */
+bool CTransfer::DoInitDownload()
+{
+	CDir dir;
+	bool b = FALSE;
+	
+	if ( eMedium == eltFILE )
+	{
+		int mode = 0;
+
+		m_File.Close();
+
+		dir.SetPath("");
+
+		if ( dir.IsFile(sSrcFilename) == FALSE )
+		{
+			// create file
+			mode |= IO_CREAT;
+		}
+
+		mode |= IO_RAW | IO_WRITEONLY;
+
+		if ( m_File.Open( sSrcFilename, mode, MO_IRUSR|MO_IWUSR|MO_IRGRP|MO_IROTH ) == FALSE )
+		{
+			CallBack_SendError(strerror(errno));
+			perror("File open");
+		}
+		else
+		{
+			if ( CConfig::Instance()->GetCreateFile() == TRUE )
+			{
+				if ( ((mode & IO_CREAT) == IO_CREAT) &&
+				     (m_nFileSize > 0) )
+				{
+					if ( m_File.Seek(m_nFileSize-1,SEEK_SET) == TRUE )
+					{
+						if ( m_File.Write("\0",1) == 1 )
+						{
+							b = TRUE;
+						}
+					}
+				}
+			}
+			else
+			{
+				b = TRUE;
+			}
+				
+			if ( b == TRUE )
+			{
+				// seek to the startposition
+				if ( m_File.Seek(m_nStartPosition,SEEK_SET) == FALSE )
+				{
+					b = FALSE;
+				}
+				else
+				{
+					b = TRUE;
+				}
+			}
+				
+			if ( b == FALSE )
+			{
+				CallBack_SendError(strerror(errno));
+				perror("File seek");
+				m_File.Close();
+			}
+		}
+	}
+	else
+	{
+		pByteArray->SetSize(0);
+		b = TRUE;
+	}
+
+	return b;
+}
+
 /** */
 int CTransfer::HandleMessage( char * c, int )
 {
 	int p=0;
 	eDCMessage type;
 	CObject * Object;
-	CDir dir;
 
 	CMessageHandler MessageHandler;
 
@@ -469,7 +560,7 @@ int CTransfer::HandleMessage( char * c, int )
 	while( (m_bIdle || (eMode == estTRANSFERUPLOAD)) &&
 	       ((type=MessageHandler.Parse(&s,p,Object=0)) != DC_MESSAGE_PARSE_ERROR) )
 	{
-//		printf("transfer: handle message: %d\n",type);
+//		DPRINTF("transfer: handle message: %d\n",type);
 
 		if ( !Object )
 		{
@@ -478,15 +569,54 @@ int CTransfer::HandleMessage( char * c, int )
 
 		switch (type)
 		{
+			case DC_MESSAGE_SENDING:
+			{
+				CMessageSending * msg = (CMessageSending*)Object;
+				
+				if ( eMode != estTRANSFERDOWNLOAD || (m_bIdle == FALSE) )
+				{
+					DPRINTF("Warning: not in downloadmode/idle FILELENGTH/LISTLEN\n");
+
+					delete Object;
+					Object = 0;
+				}
+				else
+				{
+					m_nTransfered = 0;
+
+					// update endposition
+					if ( m_nEndPosition == 0 )
+					{
+						m_nEndPosition = msg->m_nLength;
+					}
+
+					// fix chunksize
+					if ( m_nChunkSize != msg->m_nLength )
+					{
+						m_nChunkSize = msg->m_nLength;
+					}
+
+					//m_nFileSize = msg->m_nFileLength;
+
+					// start transfer
+					if ( DoInitDownload() == TRUE )
+					{
+						m_bIdle = FALSE;
+					}
+				}
+
+				printf("recv sending %lld\n",msg->m_nLength);fflush(stdout);
+				break;
+			}
+			
 			case DC_MESSAGE_FILELENGTH:
 			case DC_MESSAGE_LISTLEN:
 			{
-				bool b = FALSE;
 				CMessageFileLength * msg = (CMessageFileLength*)Object;
 
 				if ( eMode != estTRANSFERDOWNLOAD || (m_bIdle == FALSE) )
 				{
-					printf("Warning: not in downloadmode/idle FILELENGTH/LISTLEN\n");
+					DPRINTF("Warning: not in downloadmode/idle FILELENGTH/LISTLEN\n");
 
 					delete Object;
 					Object = 0;
@@ -509,77 +639,8 @@ int CTransfer::HandleMessage( char * c, int )
 
 					m_nFileSize = msg->m_nFileLength;
 
-					if ( eMedium == eltFILE )
-					{
-						int mode = 0;
-
-						m_File.Close();
-
-						dir.SetPath("");
-
-						if ( dir.IsFile(sSrcFilename) == FALSE )
-						{
-							// create file
-							mode |= IO_CREAT;
-						}
-
-						mode |= IO_RAW | IO_WRITEONLY;
-
-						if ( m_File.Open( sSrcFilename, mode, MO_IRUSR|MO_IWUSR|MO_IRGRP|MO_IROTH ) == FALSE )
-						{
-							CallBack_SendError(strerror(errno));
-							perror("File open");
-						}
-						else
-						{
-							if ( CConfig::Instance()->GetCreateFile() == TRUE )
-							{
-								if ( ((mode & IO_CREAT) == IO_CREAT) &&
-								     (m_nFileSize > 0) )
-								{
-									if ( m_File.Seek(m_nFileSize-1,SEEK_SET) == TRUE )
-									{
-										if ( m_File.Write("\0",1) == 1 )
-										{
-											b = TRUE;
-										}
-									}
-								}
-							}
-							else
-							{
-								b = TRUE;
-							}
-
-							if ( b == TRUE )
-							{
-								// seek to the startposition
-								if ( m_File.Seek(m_nStartPosition,SEEK_SET) == FALSE )
-								{
-									b = FALSE;
-								}
-								else
-								{
-									b = TRUE;
-								}
-							}
-
-							if ( b == FALSE )
-							{
-								CallBack_SendError(strerror(errno));
-								perror("File seek");
-								m_File.Close();
-							}
-						}
-					}
-					else
-					{
-						pByteArray->SetSize(0);
-						b = TRUE;
-					}
-
 					// start transfer
-					if ( b == TRUE )
+					if ( DoInitDownload() == TRUE )
 					{
 						m_bIdle = FALSE;
 
@@ -594,7 +655,7 @@ int CTransfer::HandleMessage( char * c, int )
 			{
 				if ( (eMode != estTRANSFERUPLOAD) || (m_bIdle == FALSE) )
 				{
-					printf("Warning: not in uploadmode/idle SEND [%d/%d]\n",eMode,m_bIdle);
+					DPRINTF("Warning: not in uploadmode/idle SEND [%d/%d]\n",eMode,m_bIdle);
 
 					delete Object;
 					Object = 0;
@@ -648,8 +709,7 @@ int CTransfer::HandleMessage( char * c, int )
 						m_bIdle = FALSE;
 					}
 
-					if ( dclibVerbose() > 0 )
-						printf("start upload ...'%s'\n",sSrcFilename.Data());
+					DPRINTF("start upload ...'%s'\n",sSrcFilename.Data());
 				}
 
 				break;
@@ -670,7 +730,7 @@ int CTransfer::HandleMessage( char * c, int )
 					if ( eDstDirection != msg->m_eDirection )
 					{
 						Disconnect(TRUE);
-						printf("Warning: remote will change his direction (hack) ...\n");
+						DPRINTF("Warning: remote will change his direction (hack) ...\n");
 						delete Object;
 						Object = 0;
 					}
@@ -696,7 +756,7 @@ int CTransfer::HandleMessage( char * c, int )
 				}
 				else
 				{
-					printf("Warning: Nick msg from '%s' only allowed in handshake",msg->m_sNick.Data());
+					DPRINTF("Warning: Nick msg from '%s' only allowed in handshake",msg->m_sNick.Data());
 
 					delete Object;
 					Object = 0;
@@ -720,12 +780,14 @@ int CTransfer::HandleMessage( char * c, int )
 					if ( msg->m_bExtProtocol == TRUE )
 					{
 						s = "";
-#ifdef HAVE_LIBBZ2
-						s += "BZList ";
-#endif
+						if ( CFileManager::Instance()->GetShareBufferSize(esbtBZ) > 0 )
+							s += "BZList ";
+						if ( CFileManager::Instance()->GetShareBufferSize(esbtXMLBZ) > 0 )
+							s += "XmlBZList ";
 #ifdef HAVE_SSL
-						if ( (CConfig::Instance()->GetTransferCert() != "") &&
-						     (CConfig::Instance()->GetTransferKey() != "") )
+						CDir dir;
+						if ( dir.IsFile(CConfig::Instance()->GetTransferCert(),FALSE) &&
+						     dir.IsFile(CConfig::Instance()->GetTransferKey(),FALSE) )
 						{
 							s += "SSL ";
 						}
@@ -760,7 +822,7 @@ int CTransfer::HandleMessage( char * c, int )
 			{
 				CMessageSupports * msg = (CMessageSupports*)Object;
 								
-				printf("LOCK %d %d %d\n",
+				DPRINTF("LOCK %d %d %d\n",
 				m_MessageLock.m_eClientVersion,
 				m_MessageLock.m_nVersionMajor,
 				m_MessageLock.m_nVersionMinor);
@@ -768,7 +830,7 @@ int CTransfer::HandleMessage( char * c, int )
 				// now we need to check some broken clients and limit support
 				if ( (m_MessageLock.m_eClientVersion == eucvDCPP) &&
 				     (m_MessageLock.m_nVersionMajor == 0) &&
-				     (m_MessageLock.m_nVersionMinor == 401) )
+				     (m_MessageLock.m_nVersionMinor <= 403) )
 				{
 					// disable xml list for dc++ 0.401
 					msg->m_bXMLBZList = FALSE;
@@ -792,7 +854,7 @@ int CTransfer::HandleMessage( char * c, int )
 						b = ChangeSocketMode(esmSSLSERVER, CConfig::Instance()->GetTransferCert(), CConfig::Instance()->GetTransferKey() );
 					else
 						b = ChangeSocketMode(esmSSLCLIENT);
-					printf("change to ssl mode %d\n",b);
+					DPRINTF("change to ssl mode %d\n",b);
 					delete Object;
 					Object = 0;
 				}
@@ -823,7 +885,7 @@ int CTransfer::HandleMessage( char * c, int )
 				}
 				else
 				{
-					printf("Warning: Cancel msg not in uploadmode/idle SEND [%d/%d]\n",eMode,m_bIdle);
+					DPRINTF("Warning: Cancel msg not in uploadmode/idle SEND [%d/%d]\n",eMode,m_bIdle);
 
 					delete Object;
 					Object = 0;
@@ -834,7 +896,7 @@ int CTransfer::HandleMessage( char * c, int )
 
 			default:
 			{
-				//printf("ctransfer: unknown message %d\n",type);
+				//DPRINTF("ctransfer: unknown message %d\n",type);
 			}
 		}
 
@@ -906,7 +968,7 @@ void CTransfer::DataSend()
 
 		if ( eMedium == eltFILE )
 		{
-			//printf("%llu %llu %llu %llu\n",len,lStartPosition,lChunkSize,lCurrentPosition);
+			//DPRINTF("%llu %llu %llu %llu\n",len,lStartPosition,lChunkSize,lCurrentPosition);
 
 			// load data from filehandle and send ...
 			if ( m_nFileBufferPos == m_nFileBufferSize )
@@ -972,7 +1034,7 @@ void CTransfer::DataSend()
 			len = m_nPendingSendBytes;
 		}
 
-//		printf("LEN: %d\n",len);
+//		DPRINTF("LEN: %d\n",len);
 
 		// write data to socket
 		l = Write( pByteArray->Data()+m_nFileBufferPos, (long)len, TRUE );
@@ -1018,8 +1080,7 @@ void CTransfer::DataSend()
 		     ( (m_MessageSupports.m_bChunk == TRUE) &&
 		       (m_nChunkSize == m_nTransfered) ) )
 		{
-			if ( dclibVerbose() > 0 )
-				printf("end found\n");
+			DPRINTF("end found\n");
 			if ( eMedium == eltFILE )
 			{
 				m_File.Close();
@@ -1053,14 +1114,14 @@ void CTransfer::DataTimeout()
 {
 	if ( (eMode == estTRANSFERDOWNLOAD) && (m_bIdle == FALSE) )
 	{
-		printf("CTransfer: download timeout ...\n");
+		DPRINTF("CTransfer: download timeout ...\n");
 
 		// send string ...
 		SendString("|");
 	}
 	else if ( eMode == estTRANSFERHANDSHAKE )
 	{
-		printf("CTransfer: handshake timeout ...\n");
+		DPRINTF("CTransfer: handshake timeout ...\n");
 
 		// handshake timeout
 		Disconnect(TRUE);
@@ -1086,11 +1147,11 @@ void CTransfer::DataAvailable( const char * buffer, int len )
 	i = 0;
 	p = 0;
 
-	while ( i != len )
+	while ( i < len )
 	{
 		if ( (eMode == estTRANSFERDOWNLOAD) && (m_bIdle == FALSE) )
 		{
-			while ( (i != len) && (!m_bIdle) )
+			while ( (i < len) && (!m_bIdle) )
 			{
 				if ( eMedium == eltFILE )
 				{
@@ -1121,6 +1182,7 @@ void CTransfer::DataAvailable( const char * buffer, int len )
 				// support chunks or if fileend reached
 				if ( (m_nChunkSize == m_nTransfered) &&
 				    ((m_MessageSupports.m_bChunk == TRUE) ||
+				     (m_MessageSupports.m_bZBlock == TRUE) ||
 				     ((m_nStartPosition+m_nChunkSize) == m_nFileSize) ) )
 				{
 					m_bIdle = TRUE;
@@ -1137,7 +1199,9 @@ void CTransfer::DataAvailable( const char * buffer, int len )
 				if ( m_nChunkSize == m_nTransfered )
 				{
 					// if endposition not filesize and remote don't support chunks we must disconnect
-					if ( ((m_nStartPosition+m_nChunkSize) != m_nFileSize) && (m_MessageSupports.m_bChunk == FALSE) )
+					if ( ((m_nStartPosition+m_nChunkSize) != m_nFileSize) &&
+					     (m_MessageSupports.m_bChunk == FALSE) &&
+					     (m_MessageSupports.m_bZBlock == FALSE) )
 					{
 						Disconnect();
 					}
@@ -1160,7 +1224,7 @@ void CTransfer::DataAvailable( const char * buffer, int len )
 
 		if ( i == p )
 		{
-			printf("WARNING: unknown data ! [%d %d]\n",i,len);
+			DPRINTF("WARNING: unknown data ! [%d %d]\n",i,len);
 			break;
 		}
 		else
@@ -1177,6 +1241,32 @@ int CTransfer::HandleFileTransfer( const char * buffer, int len )
 	ulonglong l;
 	CString serr="";
 	bool berr=FALSE;
+	CByteArray zbuffer;
+	int outlen;
+	int inlen;
+	
+	// decompress buffer ...
+	if ( m_nDataType == 1 )
+	{
+		zbuffer.SetSize(1024*100);
+		outlen = 1024*100;
+		inlen  = len;
+		if ( m_ZLib.InflateZBlock( (char*)buffer, &inlen, (char*)zbuffer.Data(), &outlen ) == FALSE )
+		{
+			// close file
+			m_File.Close();
+
+			CallBack_SendError(serr);
+
+			// disconnect on error
+			Disconnect();
+			
+			return -1;
+		}
+		
+		len = outlen;
+		buffer = (const char*)zbuffer.Data();
+	}
 
 	// check endposition
 	if ( (len+m_nTransfered) > m_nChunkSize )
@@ -1205,7 +1295,7 @@ int CTransfer::HandleFileTransfer( const char * buffer, int len )
 			{
 				i1 -= m_File.GetBufferPos();
 
-				printf("CTransfer::HandleFileTransfer: write failed with %d pending bytes\n",m_File.GetBufferPos());
+				DPRINTF("CTransfer::HandleFileTransfer: write failed with %d pending bytes\n",m_File.GetBufferPos());
 				berr = TRUE;
 				serr = strerror(errno);
 				break;
@@ -1232,6 +1322,13 @@ int CTransfer::HandleFileTransfer( const char * buffer, int len )
 		// disconnect on error
 		Disconnect();
 	}
+	else
+	{
+		if ( m_nDataType == 1 )
+		{
+			i1 = inlen;
+		}
+	}
 
 	return i1;
 }
@@ -1242,6 +1339,29 @@ int CTransfer::HandleBufferTransfer( const char * buffer, int len )
 	int i;
 	CString serr="";
 	bool berr=FALSE;
+	CByteArray zbuffer;
+	int outlen;
+	int inlen;
+	
+	// decompress buffer ...
+	if ( m_nDataType == 1 )
+	{
+		zbuffer.SetSize(1024*100);
+		outlen = 1024*100;
+		inlen  = len;
+		if ( m_ZLib.InflateZBlock( (char*)buffer, &inlen, (char*)zbuffer.Data(), &outlen ) == FALSE )
+		{
+			CallBack_SendError(serr);
+
+			// disconnect on error
+			Disconnect();
+			
+			return -1;
+		}
+		
+		len = outlen;
+		buffer = (const char*)zbuffer.Data();
+	}
 
 	// check endposition
 	if ( (len+m_nTransfered) > m_nChunkSize )
@@ -1262,10 +1382,9 @@ int CTransfer::HandleBufferTransfer( const char * buffer, int len )
 	{
 		if ( sDstFilename == DC_USER_FILELIST_HE3 )
 		{
-#ifdef HAVE_LIBBZ2
 			// decompress filelist
-			if ( (m_MessageSupports.m_bBZList == FALSE) && (m_MessageSupports.m_bXMLBZList == FALSE) )
-#endif
+			if ( (m_MessageSupports.m_bBZList == FALSE) &&
+			     (m_MessageSupports.m_bXMLBZList == FALSE) )
 			{
 				CHE3 *he3=0;
 				CString * s=0;
@@ -1286,7 +1405,6 @@ int CTransfer::HandleBufferTransfer( const char * buffer, int len )
 					serr = "he3 decompress failed";
 				}
 			}
-#ifdef HAVE_LIBBZ2
 			else
 			{
 				CBZ * bz = 0;
@@ -1308,9 +1426,7 @@ int CTransfer::HandleBufferTransfer( const char * buffer, int len )
 
 				delete bz;
 			}
-#endif
 		}
-
 	}
 
 	// handle error
@@ -1323,6 +1439,13 @@ int CTransfer::HandleBufferTransfer( const char * buffer, int len )
 		// disconnect on error
 		Disconnect();
 	}
+	else
+	{
+		if ( m_nDataType == 1 )
+		{
+			i = inlen;
+		}
+	}
 
 	return i;
 }
@@ -1333,12 +1456,20 @@ int CTransfer::HandleControlTransfer( const char * buffer, int len )
 	CString s;
 	int i,p;
 
-	i = p = 0;
+	p = 0;
 
-	s = sBuffer + CString().Set(buffer,len);
-
-	// search last '|'
-	i = s.FindRev('|');
+	// search first '|'
+	for(i=0;i<len;i++)
+	{
+		if ( buffer[i] == '|' )
+		{
+			s = sBuffer + CString().Set(buffer,i+1);
+			break;
+		}
+	}
+	
+	if ( i == len )
+		i = -1;
 	i++;
 
 	if ( i > 0 )
@@ -1348,7 +1479,7 @@ int CTransfer::HandleControlTransfer( const char * buffer, int len )
 		// some clients send data after filelength (e.g. nmdc 2.02)
 		if ( (p < s.Length()) && (!m_bIdle) && (eMode == estTRANSFERDOWNLOAD) )
 		{
-			printf("WARNING: transfer->client send unwanted data ! [%d %d %d]\n",i,p,len);
+			DPRINTF("WARNING: transfer->client send unwanted data ! [%d %d %d]\n",i,p,len);
 		}
 	}
 
